#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Scoary - Microbial Pan-GWAS GUI
By Ola Brynildsrud
Norwegian Institute of Public Health
olbb@fhi.no
"""

import sys, os
import threading

try:
    import Tkinter
except ImportError:
    #Python 3 issues
    try:
        import tkinter as Tkinter
    except:
        sys.exit("Need to have Tkinter / tkinter installed")
    
try:
    import tkFileDialog
except ImportError:
    # Python 3 issues
    try:
        from tkinter import filedialog
        tkFileDialog = filedialog
    except:
        sys.exit("Could not find tkFileDialog / filedialog")

try:
    import ttk
except ImportError:
    # Python 3 issues
    try:
        from tkinter import ttk as ttk
    except:
        sys.exit("Could not find ttk / tkinter.ttk")
    
try:
    ttk
    Tkinter
except NameError:
    sys.exit("Need the following installed: Tkinter, tkFileDialog, ttk")
    
try:
    import os
except ImportError:
    sys.exit("Need to be able to import os")
    
try:
    import argparse
except ImportError:
    sys.exit("Need to be able to import argparse")
    
try:
    import scoary.methods as sm
    import scoary
except ImportError:
    sys.exit("Could not find the main Scoary executable") 

from pkg_resources import resource_string, resource_filename

class ScoaryGUI(Tkinter.Tk):
    """
    Create the main GUI window
    """
    def __init__(self,parent):
        Tkinter.Tk.__init__(self,parent)
        self.parent = parent

        self.Scoary_parameters = {"GPA": None,"Trait":None,"Tree":None,
                                  "Restrict":None, "Writetree": False,
                                  "Delimiter":",", "Startcol": "15",
                                  "Maxhits":None, "Notime":False,
                                  "Outdir":None, "Permutations":0,
                                  "No_pairwise":False, "Collapse":False,
                                  "Cutoffs": {"I": [1,0.05], 
                                              "B": [0,1.0], 
                                              "BH": [0,1.0], 
                                              "PW": [0,1.0], 
                                              "EPW": [1,0.05], 
                                              "P":[0,1.0]},                                
                                }
        self.initialize_menu()
        
        self.toppart = Tkinter.Frame(self,height="350",width="800")
        self.bottompart = Tkinter.Frame(self,height="50",width="800")
        self.toppart.pack(side='top',expand=False)
        self.bottompart.pack(side='bottom',expand=True,fill='both')
        
        self.nwpart = Tkinter.Frame(self.toppart,
                                    height="350",
                                    width="250")
        self.nepart = Tkinter.LabelFrame(self.toppart,
                                         height="350",
                                         width="550",
                                         text="Control panel")
        self.nwpart.pack(side='left',expand=False)
        self.nepart.pack(side='right',expand=True)
        
        # Add further frames to top or bottom
        
        self.logocanvas = Tkinter.Canvas(self.nwpart,
                                         height="250",
                                         width="250",
                                         relief="ridge",
                                         bd=0,
                                         highlightthickness=0)
        self.logocanvas.pack(side='top',expand=False)
        
        self.citationframe = Tkinter.Frame(self.nwpart,
                                           height="100",
                                           width="250")
        self.citationframe.pack(side='bottom',expand=False,fill='none')
        
        self.initialize_controlboard()
        self.initialize_logo()
        self.initialize_citation()
        self.initialize_statusframe()
        
        ######## MENUS ########
        
    def initialize_menu(self):
        """
        Initialize the menu at the top
        """
        self.menubar = Tkinter.Menu(self,relief="flat")
        filemenu = Tkinter.Menu(self.menubar,tearoff=0)
        filemenu.add_command(label="About",command=self.AboutScoary)
        filemenu.add_separator()
        filemenu.add_command(label="Quit",command=self.quit)
        self.filemenu = filemenu
        self.menubar.add_cascade(label="File",menu=self.filemenu)
        
        optsmenu = Tkinter.Menu(self.menubar,tearoff=0)
        optsmenu.add_command(label="Clear", command=self.ClearAll)
        optsmenu.add_command(label="Test example",
                             command=self.TestExample)
        self.optsmenu = optsmenu
        self.menubar.add_cascade(label="Options",menu=self.optsmenu)
        
        self.config(menu=self.menubar)        
        
    def initialize_citation(self):
        """
        Initialize the citation frame - below the logo
        """
        myfontstyle = ("Arial",8)
        self.citation = Tkinter.Label(self.citationframe,
                                      text=self.citationtext(),
                                      anchor='center',
                                      justify='center',
                                      font=myfontstyle)
        self.citation.pack(expand=True)
        
    def initialize_statusframe(self):
        """
        Initialize the frame and statusbar occupying the bottom
        """
        frame = self.bottompart
        
        frame.pb = ttk.Progressbar(frame,
                                   orient='horizontal',
                                   mode='determinate',
                                   maximum=100)
        frame.pb.pack(fill='both',expand=True,side='top')

        frame.lab = Tkinter.Label(frame,text=u"Awaiting input options")
        frame.lab.pack(in_=frame.pb,expand=True)
        #sys.stdout = StdoutToLabel(frame.lab, progressbar=frame.pb)
        sys.stdout = StdoutToLabel(frame.lab,
                                   progressbar=frame.pb,
                                   width=frame.cget('width'))
    
    def initialize_controlboard(self):
        """
        Initialize the controlboard - where all the settings are set.
        """
        board = self.nepart
        
        self.GPAentryVariable = Tkinter.StringVar()
        board.GPAentry = \
            Tkinter.Entry(board, 
            textvariable = self.GPAentryVariable,
            width=60)
        board.GPAentry.grid(column=0,row=0,sticky='W',columnspan=2)
        self.GPAentryVariable.set("Path to gene presence absence file")
        
        self.TraitsentryVariable = Tkinter.StringVar()
        board.Traitsentry = \
            Tkinter.Entry(board, 
            textvariable=self.TraitsentryVariable,
            width=60)
        board.Traitsentry.grid(column=0,row=1,sticky='W',columnspan=2)
        self.TraitsentryVariable.set("Path to traits/phenotype file")
        
        self.TreeentryVariable = Tkinter.StringVar()
        board.Treeentry = \
            Tkinter.Entry(board, 
            textvariable=self.TreeentryVariable,
            width=60)
        board.Treeentry.grid(column=0,row=2,sticky='W',columnspan=2)
        self.TreeentryVariable.set(
            "(Optional) Path to custom tree file")
        
        self.RestrictVariable = Tkinter.StringVar()
        board.Restrictentry = \
            Tkinter.Entry(board, 
            textvariable=self.RestrictVariable,
            width=60)
        board.Restrictentry.grid(column=0,row=3,sticky='W',columnspan=2)
        self.RestrictVariable.set(
            "(Optional) Path to file naming isolates to include")
        
        self.Outputdir = Tkinter.StringVar()
        board.Outputentry = \
            Tkinter.Entry(board, 
            textvariable=self.Outputdir,
            width=60)
        board.Outputentry.grid(column=0,row=4,sticky='W',columnspan=2)
        self.Outputdir.set("(Optional) Output directory")

        browsebuttonGPA = \
            Tkinter.Button(board, 
            text=u"Browse...", 
            command=self.BrowseButtonClickGPA)
        browsebuttonGPA.grid(column=2,row=0,sticky='e')
        
        browsebuttonTraits = \
            Tkinter.Button(board,
            text=u"Browse...", 
            command=self.BrowseButtonClickTraits)
        browsebuttonTraits.grid(column=2,row=1,sticky='e')
        
        browsebuttonTreeFile = \
            Tkinter.Button(board,
            text=u"Browse...", 
            command=self.BrowseButtonClickTreeFile)
        browsebuttonTreeFile.grid(column=2,row=2,sticky='e')
        
        browsebuttonRestrict = \
            Tkinter.Button(board,
            text=u"Browse...",
            command=self.BrowseButtonClickRestrict)
        browsebuttonRestrict.grid(column=2,row=3,sticky='e')
        
        browsebuttonOutput = \
            Tkinter.Button(board,
            text=u"Browse...",
            command=self.BrowseButtonClickOutput)
        browsebuttonOutput.grid(column=2,row=4,sticky='e')
        
        # Initialize frame for cutoffs
        board.pframe = Tkinter.LabelFrame(board,
                                          text="Cut-offs",
                                          relief='ridge')
        board.pframe.grid(column=0,row=5,sticky='w')
        self.initialize_pvalueframe()
        
        # Initialize frame for misc options
        board.miscframe = Tkinter.LabelFrame(board,
                                             text="Misc options",
                                             relief='ridge')
        board.miscframe.grid(column=1,row=5,sticky='e',columnspan=2)
        self.initialize_miscopts()
        
        ## Create extra space
        
        board.emptyspace = Tkinter.Frame(board)
        board.emptyspace.grid(column=0,row=6,columnspan=3,pady=(20,20))
        
        masterbuttonfont = ("Courier", 16)
        
        RunButton = Tkinter.Button(board.emptyspace,
            text=u"Run analysis",
            font=masterbuttonfont,
            command=self.RunAnalysis,
            padx=15,pady=10)
        RunButton.grid(column=2,row=0)
        
        QuitButton = Tkinter.Button(board.emptyspace,
            text=u"Quit",
            font=masterbuttonfont,
            command=self.quit,padx=15,pady=10)
        QuitButton.grid(column=1,row=0)
        
        HelpButton = Tkinter.Button(board.emptyspace,
                                    text=u"Help",
                                    font=masterbuttonfont,
                                    command=self.HelpButton,
                                    padx=15,pady=10)
        HelpButton.grid(column=0,row=0)
        
    def initialize_miscopts(self):
        """
        Initialize the miscellaneous options
        """
        board = self.nepart
        miscframe = board.miscframe
        
        # Max hits
        self.mhtext = Tkinter.StringVar()
        miscframe.mhlab = Tkinter.Label(miscframe,
                                        textvariable=self.mhtext)
        miscframe.mhlab.grid(column=0,row=0,sticky='w')
        self.mhtext.set("Max hits")
        
        self.maxhitsvar = Tkinter.StringVar()
        miscframe.maxhitsentry = \
            Tkinter.Entry(miscframe,
            textvariable=self.maxhitsvar,
            width=8)
        miscframe.maxhitsentry.grid(column=1,row=0)
        self.maxhitsvar.set("")
        
        # Delimiter
        self.delimtext = Tkinter.StringVar()
        miscframe.delim = \
            Tkinter.Label(miscframe,
            textvariable=self.delimtext)
        miscframe.delim.grid(column=0,row=1,sticky='w')
        self.delimtext.set("Delimiter")
        
        self.delimvar = Tkinter.StringVar()
        miscframe.delimentry = \
            Tkinter.Entry(miscframe,
            textvariable=self.delimvar,
            width=8)
        miscframe.delimentry.grid(column=1,row=1)
        self.delimvar.set(",")
        
        # Starting column
        self.sctext = Tkinter.StringVar()
        miscframe.sclab = \
            Tkinter.Label(miscframe,
            textvariable=self.sctext)
        miscframe.sclab.grid(row=2,column=0,sticky='w')
        self.sctext.set("Startcol GPA file")
        
        self.scvar = Tkinter.StringVar()
        miscframe.sc = Tkinter.Entry(miscframe,textvariable=self.scvar,width=8)
        miscframe.sc.grid(row=2,column=1)
        self.scvar.set("15")
        
        # Permutations
        self.permtext = Tkinter.StringVar()
        miscframe.permlab = \
            Tkinter.Label(miscframe,
            textvariable=self.permtext)
        miscframe.permlab.grid(row=3,column=0,sticky='w')
        self.permtext.set("Permutations")
        
        self.permvar = Tkinter.StringVar()
        miscframe.perm = \
            Tkinter.Entry(miscframe,
            textvariable=self.permvar,
            width=8)
        miscframe.perm.grid(row=3,column=1)
        self.permvar.set("0")
        
        # No timestamp
        self.notimevar = Tkinter.IntVar()
        miscframe.notime = \
            Tkinter.Checkbutton(miscframe,
            text=u"No timestamp",
            onvalue=1,
            offvalue=0,
            variable=self.notimevar)
        miscframe.notime.grid(row=5,column=0,sticky='w')
        
        # Write tree
        self.writetreevar = Tkinter.IntVar()
        miscframe.writetree = \
            Tkinter.Checkbutton(miscframe,
            text=u"Write tree",
            onvalue=1,
            offvalue=0,
            variable=self.writetreevar)
        miscframe.writetree.grid(row=6,column=0,sticky='w')

        # No_pairwise
        self.nopairwisevar = Tkinter.IntVar()
        miscframe.nopairwise = \
            Tkinter.Checkbutton(miscframe,
            text=u"No pairwise",
            onvalue=1,
            offvalue=0,
            variable=self.nopairwisevar)
        miscframe.nopairwise.grid(row=7,column=0,sticky='w')

        # Collapse
        self.collapsevar = Tkinter.IntVar()
        miscframe.collapse = \
            Tkinter.Checkbutton(miscframe,
            text=u"Collapse corr",
            onvalue=1,
            offvalue=0,
            variable=self.collapsevar)
        miscframe.collapse.grid(row=8,column=0,sticky='w')
        
    def initialize_pvalueframe(self):
        """
        Initialize the filtration controlboard
        """
        board = self.nepart
        pframe = board.pframe
        
        # Add p-value checkboxes
        
        self.naivetext = Tkinter.StringVar()
        self.pVar = Tkinter.IntVar()
        self.pVar.set(1)
        self.pBVar = Tkinter.IntVar()
        self.pBHVar = Tkinter.IntVar()
        self.pstext = Tkinter.StringVar()
        self.pPWVar = Tkinter.IntVar()
        self.pEPWVar = Tkinter.IntVar()
        self.pEPWVar.set(1)
        self.pPermVar = Tkinter.IntVar()
        
        pframe.naivelab = \
            Tkinter.Label(pframe,
            textvariable=self.naivetext)
        self.naivetext.set(u"Pop structure-naive filters")
        pframe.pNcheck = \
            Tkinter.Checkbutton(pframe,
            text=u"Naive (Fisher's)",
            onvalue=1,
            offvalue=0,
            variable=self.pVar)
        pframe.pBcheck = \
            Tkinter.Checkbutton(pframe,
            text=u"Bonferroni",
            onvalue=1,
            offvalue=0,
            variable=self.pBVar)
        pframe.pBHcheck = \
            Tkinter.Checkbutton(
            pframe,
            text=u"Benjamini-Hochberg",
            onvalue=1,
            offvalue=0,
            variable=self.pBHVar)
        pframe.pslab = \
            Tkinter.Label(pframe,
            textvariable=self.pstext)
        self.pstext.set(u"Pop structure-aware filters")
        pframe.pPWcheck = \
            Tkinter.Checkbutton(pframe,
            text=u"Pairwise comparison (Best)",
            onvalue=1,
            offvalue=0,
            variable=self.pPWVar)
        pframe.pEPWcheck = \
            Tkinter.Checkbutton(pframe,
            text=u"Pairwise comparison (Entire)",
            onvalue=1,
            offvalue=0,
            variable=self.pEPWVar)
        pframe.pPermcheck = \
            Tkinter.Checkbutton(pframe,
            text=u"Empirical p-value (Permutation)",
            onvalue=1,
            offvalue=0,
            variable=self.pPermVar)
        
        pframe.naivelab.grid(column=0,row=0,sticky='w')
        pframe.pNcheck.grid(column=0,row=1,sticky='w')
        pframe.pBcheck.grid(column=0,row=2,sticky='w')
        pframe.pBHcheck.grid(column=0,row=3,sticky='w')
        pframe.pslab.grid(column=0,row=4,sticky='w')
        pframe.pPWcheck.grid(column=0,row=5,sticky='w')
        pframe.pEPWcheck.grid(column=0,row=6,sticky='w')
        pframe.pPermcheck.grid(column=0,row=7,sticky='w')
        
        # Add p-value entry cells
        
        self.pNaive = Tkinter.StringVar()
        self.pBonf = Tkinter.StringVar()
        self.pBH = Tkinter.StringVar()
        self.pPW = Tkinter.StringVar()
        self.pEPW = Tkinter.StringVar()
        self.pPerm = Tkinter.StringVar()
        
        pframe.pNaiveEntry = Tkinter.Entry(pframe,
                                           textvariable=self.pNaive,
                                           width=8)
        pframe.pBonfEntry = Tkinter.Entry(pframe,
                                          textvariable=self.pBonf,
                                          width=8)
        pframe.pBHEntry = Tkinter.Entry(pframe,
                                        textvariable=self.pBH,
                                        width=8)
        pframe.pPWEntry = Tkinter.Entry(pframe,
                                        textvariable=self.pPW,
                                        width=8)
        pframe.pEPWEntry = Tkinter.Entry(pframe,
                                         textvariable=self.pEPW,
                                         width=8)
        pframe.pPermEntry = Tkinter.Entry(pframe,
                                          textvariable=self.pPerm,
                                          width=8)
        
        pframe.pNaiveEntry.grid(column=1,row=1,sticky='w')
        pframe.pBonfEntry.grid(column=1,row=2,sticky='w')
        pframe.pBHEntry.grid(column=1,row=3,sticky='w')
        pframe.pPWEntry.grid(column=1,row=5,sticky='w')
        pframe.pEPWEntry.grid(column=1,row=6,sticky='w')
        pframe.pPermEntry.grid(column=1,row=7,sticky='w')
        
        self.pNaive.set("0.05")
        self.pBonf.set("1.0")
        self.pBH.set("1.0")
        self.pPW.set("1.0")
        self.pEPW.set("0.05")
        self.pPerm.set("1.0")
        
        ######## EVENTS ########
        
    def AboutScoary(self):
        """
        Placeholder button. Planned short information about the method
        """
        topwin = Tkinter.Toplevel(self)
        button = \
            Tkinter.Button(topwin,
            text=str("https://github.com/AdmiralenOla/Scoary"))
        button.pack()
        
    def BrowseButtonClickGPA(self):
        """
        Browse button for gene presence absence field
        """
        myfile = \
            tkFileDialog.askopenfilename(
            filetypes=[('comma-separated values', '.csv'),
                       ('all files','.*')])
        self.GPAentryVariable.set(myfile)
        
    def BrowseButtonClickTraits(self):
        """
        Browse button for traits field
        """
        myfile = \
            tkFileDialog.askopenfilename(
            filetypes=[('comma-separated values', '.csv'),
                       ('all files','.*')])
        self.TraitsentryVariable.set(myfile)
        
    def BrowseButtonClickTreeFile(self):
        """
        Browse button for tree field
        """
        myfile = \
            tkFileDialog.askopenfilename(
            filetypes=[('newick tree files', '.nwk'),
                       ('all files','.*')])
        self.TreeentryVariable.set(myfile)
        
    def BrowseButtonClickRestrict(self):
        """
        Browse button for isolate restriction field
        """
        myfile = \
            tkFileDialog.askopenfilename(
            filetypes=[('comma-separated values','.csv'),
                       ('all files','.*')])
        self.RestrictVariable.set(myfile)            
        
    def BrowseButtonClickOutput(self):
        """
        Browse button for choosing output dir
        """
        mydir = tkFileDialog.askdirectory(mustexist=True)
        self.Outputdir.set(mydir)
    
    def HelpButton(self):
        """
        Placeholder button. Redirects to website
        """
        print("Visit https://github.com/AdmiralenOla/Scoary for help")
        
    def ClearAll(self):
        """
        Sets all variables to defaults
        """
        self.GPAentryVariable.set("Path to gene presence absence file")
        self.TraitsentryVariable.set("Path to traits/phenotype file")
        self.TreeentryVariable.set("(Optional) Path to custom tree "
                                   "file")
        self.RestrictVariable.set("(Optional) Path to file naming "
                                  "isolates to include")
        self.Outputdir.set("")
        self.maxhitsvar.set("")
        self.delimvar.set(",")
        self.scvar.set("15")
        self.notimevar.set(0)
        self.writetreevar.set(0)
        self.permvar.set("0")
        self.pVar.set(1)
        self.pBVar.set(0)
        self.pBHVar.set(0)
        self.pPWVar.set(0)
        self.pEPWVar.set(1)
        self.pNaive.set("0.05")
        self.pBonf.set("1.0")
        self.pBH.set("1.0")
        self.pPW.set("1.0")
        self.pEPW.set("0.05")
        self.pPerm.set("1.0")
        self.nopairwisevar.set(0)
        
    def TestExample(self):
        """
        Sets all variables corresponding to --test in the methods script
        """
        self.GPAentryVariable.set(
            str(os.path.join(resource_filename(__name__, 'exampledata'),
                             'Gene_presence_absence.csv')))
        self.TraitsentryVariable.set(
            str(os.path.join(resource_filename(__name__, 'exampledata'),
                             'Tetracycline_resistance.csv')))
        self.TreeentryVariable.set("")
        self.RestrictVariable.set("")
        self.Outputdir.set("./")
        self.maxhitsvar.set("")
        self.delimvar.set(",")
        self.scvar.set("15")
        self.notimevar.set(0)
        self.writetreevar.set(0)
        self.permvar.set("0")
        self.pVar.set(1)
        self.pBVar.set(0)
        self.pBHVar.set(0)
        self.pPWVar.set(0)
        self.pEPWVar.set(1)
        self.pNaive.set("0.05")
        self.pBonf.set("1.0")
        self.pBH.set("1.0")
        self.pPW.set("1.0")
        self.pEPW.set("0.05")
        self.pPerm.set("1.0")
        self.nopairwisevar.set(0)
        
        ######## RUNNING THE ANALYSIS ########
        
    def RunAnalysis(self):
        """
        Upon click "Run analysis" - Reads all the set parameters and 
        calls run method
        """
        self.Scoary_parameters["GPA"] = self.GPAentryVariable.get()
        self.Scoary_parameters["Traits"] = \
            self.TraitsentryVariable.get()
        self.Scoary_parameters["Tree"] = self.TreeentryVariable.get()
        self.Scoary_parameters["Outdir"] = self.Outputdir.get()
        self.Scoary_parameters["Restrict"] = self.RestrictVariable.get()
        self.Scoary_parameters["Writetree"] = self.writetreevar.get()
        self.Scoary_parameters["Delimiter"] = self.delimvar.get()
        self.Scoary_parameters["Startcol"] = self.scvar.get()
        self.Scoary_parameters["Maxhits"] = self.maxhitsvar.get()
        self.Scoary_parameters["Notime"] = self.notimevar.get()
        self.Scoary_parameters["Permutations"] = self.permvar.get()
        self.Scoary_parameters["No_pairwise"] = self.nopairwisevar.get()
        self.Scoary_parameters["Collapse"] = self.collapsevar.get()
        self.Scoary_parameters["Cutoffs"]["I"][0] = self.pVar.get()
        self.Scoary_parameters["Cutoffs"]["I"][1] = self.pNaive.get()
        self.Scoary_parameters["Cutoffs"]["B"][0] = self.pBVar.get()
        self.Scoary_parameters["Cutoffs"]["B"][1] = self.pBonf.get()
        self.Scoary_parameters["Cutoffs"]["BH"][0] = self.pBHVar.get()
        self.Scoary_parameters["Cutoffs"]["BH"][1] = self.pBH.get()
        self.Scoary_parameters["Cutoffs"]["PW"][0] = self.pPWVar.get()
        self.Scoary_parameters["Cutoffs"]["PW"][1] = self.pPW.get()
        self.Scoary_parameters["Cutoffs"]["EPW"][0] = self.pEPWVar.get()
        self.Scoary_parameters["Cutoffs"]["EPW"][1] = self.pEPW.get()
        self.Scoary_parameters["Cutoffs"]["P"][0] = self.pPermVar.get()
        self.Scoary_parameters["Cutoffs"]["P"][1] = self.pPerm.get()
        
        self.PrepareScoaryCMDline()
        
    def PrepareScoaryCMDline(self):
        """
        Prepares arguments to correspond with argparse namespace and 
        runs. Listens to sys.stdout and updates statusbar.
        """
        citation=False
        RunScoary = True
        correction = []
        p_value_cutoff = []
        for m in self.Scoary_parameters["Cutoffs"]:
            if self.Scoary_parameters["Cutoffs"][m][0] == 1:
                correction.append(m)
                try:
                    p_value_cutoff.append(
                        float(self.Scoary_parameters["Cutoffs"][m][1]))
                except ValueError:
                    print("Please enter real numbers in the p value "
                          "fields")
                    RunScoary = False
                    break
                    
        delimiter = self.Scoary_parameters["Delimiter"]
        genes = (self.Scoary_parameters["GPA"] if 
                 self.Scoary_parameters["GPA"] not in 
                 ["","Path to gene presence absence file"] else None)
        try:
            max_hits = (int(self.Scoary_parameters["Maxhits"]) if 
                        self.Scoary_parameters["Maxhits"] not in [""]
                        else None)
        except ValueError:
            print("Please enter a real number (or nothing) in the max "
                  "hits field")
            max_hits = None
            RunScoary = False
        try:    
            permutations = (abs(int(
                            self.Scoary_parameters["Permutations"])) if 
                            self.Scoary_parameters["Permutations"] 
                            not in [""]
                            else 0)
        except ValueError:
            print("Please enter a real number (or nothing) in the max "
                  "hits field")
            permutations = 0
            RunScoary = False
        newicktree = (self.Scoary_parameters["Tree"] if 
                      self.Scoary_parameters["Tree"] not in 
                      ["", "(Optional) Path to custom tree file"] 
                      else None)
        no_time = (True if self.Scoary_parameters["Notime"] == 1 
                   else False)
        outdir = (self.Scoary_parameters["Outdir"] if 
                  self.Scoary_parameters["Outdir"] not in 
                  ["","(Optional) Output directory"]
                  else "./")
        restrict_to = (self.Scoary_parameters["Restrict"] if 
                       self.Scoary_parameters["Restrict"] not in 
                       ["","(Optional) Path to file naming isolates to "
                       "include"] 
                       else None)
        start_col = self.Scoary_parameters["Startcol"]
        test = False
        threads = 1
        traits = (self.Scoary_parameters["Traits"] if 
                  self.Scoary_parameters["Traits"] not in 
                  ["","Path to traits/phenotype file"] 
                  else None)
        upgma_tree = (True if self.Scoary_parameters["Writetree"] == 1 
                      else False)
        write_reduced=False
        collapse= (True if self.Scoary_parameters["Collapse"] == 1
                   else False)
        no_pairwise = (True if self.Scoary_parameters["No_pairwise"] == 1
                       else False)
               
        myargs = argparse.Namespace(citation=citation,
                                    correction=correction,
                                    p_value_cutoff=p_value_cutoff,
                                    delimiter=delimiter,
                                    genes=genes,
                                    grabcols=[],
                                    max_hits=max_hits,
                                    newicktree=newicktree,
                                    no_time=no_time,
                                    restrict_to=restrict_to,
                                    outdir=outdir,
                                    permute=permutations,
                                    start_col=start_col,
                                    test=test,
                                    threads=threads,
                                    traits=traits,
                                    upgma_tree=upgma_tree,
                                    write_reduced=write_reduced,
                                    collapse=collapse,
                                    no_pairwise=no_pairwise)
        
        if RunScoary:
            try:
                
                sm.main(args=myargs,
                        cutoffs=dict(
                            list(zip(correction, p_value_cutoff))),
                        statusbar=sys.stdout)

            except SystemExit as SE:
                # Set status bar color to red?
                if str(SE) == "0":
                    print("Analysis complete!")
                else:
                    print("Fatal error: %s" % str(SE))
        
        # Listen to stdout and update statusbar
        
    ######## MISC METHODS ########
        
    def initialize_logo(self):
        """
        Initialize logo
        """
        photo=Tkinter.PhotoImage(data=self.Photobase64())
        self.logocanvas.img = photo
        self.logocanvas.create_image(0,0,anchor='nw',image=photo)
        
    def citationtext(self):
        """
        Returns citation info
        """
        text = "SCOARY version %s \n\n" \
        "Please cite as: \n" \
        "Brynildsrud O, Bohlin J, Scheffer L, \n" \
        "Eldholm V. Rapid scoring of genes in \n" \
        "microbial pan-genome-wide association \n" \
        "studies with Scoary. Genome Biol. \n" \
        "2016;17:238" % scoary.__version__
        return text
        
    def Photobase64(self):
        """
        base64 encoding of the logo
        """
        photo= """
        R0lGODlh+gD6AOf/AAABAAEEAAMGAgcJBQoMCAwPCw8RDhIUERYYFhkbGRwdGx8g
        HiIkISYoJSstKjsrLkArL0gqJ00pI0QsJzEyMEQtLT8wMkYvL0IxLkUwM0svMDQ2
        M0IyNE0wMUQzMEMzNUoyMjY4NUQ1N0Y1MkI3ODk7OEc3OUk5PEU7PD4/PU07OUU9
        QkE/Q04+QD9DRUFDQERETVBBSEtDSE1DQ0dFSEZHRVRDRlZDQUFJUUlLSD1PWVRK
        SjhRWkxOSzRVY1xMUEdSXj5VZFBST0BWYFNVUjRcbl9VWFZYVWJVUT9ebEtdaDZj
        dVtdWkBjd2lcVyNsh2hcXSxrgV9hXjlsg2VnZDV0inFmZzF3kmpsaXZpZyl8lh5/
        nit9nm5wbSeEo3hzcjiCnXN1ch6JroBzcnZ4dYF2di+KqRaQuoB4cnh6dyeOs4d5
        dBiUuASaw3x+e4CBf4mBeyyXwACj0wCh8YyFgIWHhJCEhBugygCm8ACo8QCp8h6l
        zhGo2ACr7h+i7QCr9IuNigCu4wCw3gqt8I6QjRyr2wCy9BCu8QC16ZqSjJKUkQC2
        8Q2z6QC2+Bew8wC59AC67wC6/Be166GWip6XkQC895eZlgC9/iC18qSclSa49QjB
        9qKdnJ2fnBDA+yi78aWgny2+9KKkoTLA9qWnpKynpamrqEfD9Kyuq7OtqjvJ+EvG
        97iysbK0sE7M9lvK9be5tl/N+bu9umDS9768wMS8tWzQ9sDCv8bBwHvT+3rW94XW
        +MbIxM/HwHza+4fY+snLyM7KzIjd95Hb+M3PzJXf/J3e/NfSytjS0dPV0ZHl+aXg
        +drWx57j+tXX1Kjj/N3a0dnb2Kjn+bLo+7nn++Te2t7h3ePi2bvu+8Ps+uPl4ubm
        3Mbv/c3t/u7o5+jq5svy+uzr4tTx++3s4+zu6/Pt7Nj0/+7w7d7z//Hw5t32++T1
        /PL08Pjz8ff27ej5//T38/D4/+z6+/P4++77/PX6/fj69/r8+ff9//n+///9+/z+
        +//9////9v7//P///yH+C1Njb2FyeSBsb2dvACH5BAEKAP8ALAAAAAD6APoAAAj+
        AP0JHEiwoMGDCBMqXMiwocOHECNKnEixosWLGDNq3Mixo8ePIEOKHEmypMmTKFOq
        XMmypcuXMGPKnEmzps2bOHPq3Mmzp8+fQIMKHUq0qNGjSJMqXcq0qdOnUKNKnUq1
        qtWrWLNq3cq1q9evYMOKHUu2rNmzaNOqXcu2rdu3cOPKnUu3rt27ePPq3cu3r9+/
        gAMLHky4sOHDiBMrXsy4sePHkCNLnky5suXLmDNr3sy5s+fPoEOLHk26tOnTqFOr
        Xs26tevXsGPLnk27tu3buHPr3s27t+/fwIMLH068uPHjyF2fs5asebRv9pKTZJcs
        DIMA2LNrT0FIGzzpHJP+MRigvbz5AANysAN/EZiD8/DP17jHfuKb+PjPt6oPkUj+
        /+URwl9D/gFoIHaADKgQIQc2GIA1CiJEnoMGDhBdhARhAeABCjDQQAMLHGAgKhgO
        xM5/adhDX0H21AFgiQIRgx8V6ynEzgv5yQKji/C98BAC+IUBYw3x1djQN/hRAGMD
        8A3w3UMixudTO8yUYscaaHBSyzeNXQffkw4Bgt9EtwjBwAETYjeAAQykIIpGyJSx
        gwkcWFDBnRVAwEELUICS0TdhOKBAAeatqYADZHDpkD2iNOpoo6QQZM0GBmRXAANk
        +APLo5zq+BCjnD4K5kMlxFcHRCeqWQCHHjZwzqf+dQBpIBXaVMQKEnp+4IEJW6hx
        hhpiPOGDCXeaYIeiEqGygYMhwNIQOVIKdF98C8R3AES8xEfARBrG5yxKt1CIXScS
        VWMEBLp64MEFUTyCybuLaBJJJW0EQacFqUTETgriYifEhQmdA9+2/qSRnyX4RfNQ
        t+dFKpEs+flo0hH9YlcDRKVA4AEHHnzAwQcRMPLuyCRjEsgFHO8wqkLwEFAxdtcq
        JPB525rynzVMxHexQ/ENQJE2ADagyEilvhyAxAyVYYHH6m7sQRWNlFyyJyw03UI1
        DNlTqdEBMCDzwP6k2aQ/EDcJsEKwxOdARV4CGEIannJEMdcBYMHQGhyry/T+BxUE
        InXJl+igLscmQLMQFXRj9+1BM5tHwHv5kRt2fBAylDN8+/0srgN1EEMORgwmHoDC
        CSXCgQkngHABBhtzMIW7f5NcxMdNm1BOQsCIHoCSCDVO4QArhn4eEw3J6vhFOFas
        gChnR/QfEbewYw8wl1ubUCpLM7KIIHFUAQQJHwgSe8lcYMDB+efP0PxAycNXQivn
        2HMOMG8Qit/6Avnu4AIDAR0f/gRJRnykgJFqGU0BQ4vIsuLDBCMNBB7VO0/cCJIO
        XfFhEfB6hAaDMATYjQ8TYrhA61qXiIPYw1rIKkjZ4MOL3h2oB0domyUIYjzz3GIh
        RILPDS/yDZdxbQD+XQCgQaAVn0wlpGjnycFBoFCBKmBQaoHAgBw++C45VGCEG3vA
        MQyCOPhUDiE5iI/DDKK/8xRgcf74Rh18RpAVmsduComSeQqgEXIoQHSmaAiPzrO2
        hYitPM2DBgdo4Inx6SAJlfjgInBAAx644AQXiCQHsmCQHjgAAX98w0KsER8BMe4/
        r2qI/c6jECKeJ0EbCWPikJYQ/HwxIQvMzgAWEIYdEuQHFVDDByURgSnGbhEskEEh
        NYiIO1QBBQ+4hkKIQQgmJOCVCImPJj+Jnxk6hF+YS0gXz0O6jWgDm1xrgEIEGJ8V
        KQQQC+gCLKxBDgB64wM28ODfHuEDGmxhD5D+8MQlNAEvGKzgiSNbxCO0UAFKauSP
        2TEiGfMDEXKehwgIuUd86PiRZKQBoRSCKEJsBh8hcAQUHvBC1Mb3CB1w4AIV6IAM
        dLCEK6gBCCwAaMk+AYIWaEQU8RESNeFzhIi0rTwGQIhDzUMikciCCAbs15sOYjD4
        jDEjNsBAIajIB5R5DH3nqwALChm7SyTBAtugCHVIkcOcujA+E2yIG+LTQoPssTwW
        OgkpQoDR/LDRIKo8T600wo4HmECmUlsEDAbntI95zAWJHN8WLJCJT8HDokn9j04X
        SrmI+O88IThIXb2WEnt8AxY9aFAaDgI5Um4kGBCQgTxLtgg+5I2weeP+AApWW7I4
        VMAIDDnHGxiwNQcptCBl1E4KHYLE8hSgebnLpkuIEav/cLYgNSwPR2phAQxo4RKx
        ewQNnJau2J7Ub+MLxARkkJBoUCGy4hrtTs8TSoi4sTzdFAgZ4EPRmHSit/A5iA9N
        q5FSnG8CW6AtJtqAAY/p6nwdO98HMNAGwJJsERH4wPq0IYTEqZeyX5JIquRTECY9
        9CP3UNFE7hgfWw6krgHgSCkK7IEKcEGmi7gEED5wVSxe9QJPEDC8IsABB/ojXA7a
        r3nccFb4mBMiESyPkZCkQ4xYAxaK6EIOGICAAxBgmhHhZHyIXBA5mkfFKHMaGC7h
        iUcgggtNuMD+gZsW26YBwcEB5bEDSdGgDXTic0LWDpaBO6aJBBc7bfXHNssTM4vk
        1TyshAgF4tPTgvy0PHvNCCvQ57EL6IAFHZAADXwAAsMa9qoI/gAJ4PwuRkzABObc
        cM+kQIpkFCTP2dkzQf6MnYp4WTsEFAiJzXOqiwjPPEGdiBTik2h/gNM8edQILljM
        MQ4sQQxyWIQ+2wACpxG2sB1jBD9Z+wgvVCAGBInleYQQaIPUVdYDoXWKKdJU054Q
        Pq6+yFDNk7mIVBg+SizIoMuTa4ZYwnM+lpTGBjdqqe1BBE3bmK6YtisQsGERAF3E
        HVBgvi88ED9pNYipilwoi8Qnj9M6Xkb+8IOAicA6O1QwSCd6xjNVUaAO8R6IPR7Q
        7AuYIXZ76PRVQX2BKkjCDCBYgRo0eIcVINgCtBiIIuKzVIVsfL3GtUj7ygPRj2uk
        rOcp6kNWLkaDZCs+n1sIKhhNECs4jQU6fkQbwtxsDWghEO56BCPAAAIc6MCq5xOB
        OwYi7vKEPSGqLg8qMWyeQk+EzucZQOBlGXCKIAw/Mc/tKM+zvnfDZ/CwfLpAguGx
        Ctx8fI2g9rpMEAVErFYTnwiCgvP2A4Lg58gHwSl8ek348pTc4zyFz3MxsnjzAONI
        +VEAQu7tRRvhJ4XsoBMMuPrBO3igCZAY6d/ioOaPcQACjR0Iftr+i5Bdmwfd+YvP
        7SsSBgfVOyNd+E8IhOgPhg0QIVqGT7BNeHLslMAgdMhAFLb9QT54QGQfxAhs9wEn
        0A+ux1YK8XXwwWW1px3jRxEyciDzpxFM9h8NIAR1IAqmIAp1QASLBiBChF+OIzkE
        AQzeR28GYQ8qkAFTpUh3EAM6NjKecAKDkwGUUBClZR4PqEL50QUcZ3sYUX9J5BFv
        pTvZcWEHEYH4cQBSAAiKQAY5aEYIQQkQgAMxiAmXEAVKcIWYUAk4gD4zAHtFWB4h
        8HcC8Q3plx/EA3UOiBGIByAmthEeZoTYQTAKgXVcE18FAQU9F4OXMARewH/ZpXoe
        s0UFAQ/+/4EFvJAMqBBaAFJs4QcfOzgR7IBi2jGBHSGERgNNCBFdFeNRCREOD/AB
        XqBjlRABe0BqJDMFHoABk9B9dCh8bJgdkzgRjwYf4EeBltgvxOAQFfgyvKMQx/AA
        F3AF2EUyj7AIRWBzV9haK2ABY5AQl/U7/zOL2FGLEqGAkCcS7PCBdHMAvfgQ5JAA
        L3N/DdELHFABS/BEjxAIT9Bp6SgGphhCDxCNCyIuFGB55uFJfCaJGuGJhFYSYbCL
        ANIDFHFsDbKGDnEMLSBb0aYFEZBg6jIBQ1cycrcEIgQHeuQgoOiI5iFODUiLGpFk
        5vFUI5GG4iIEKxMRyTB5/5EAw9X+EPAABQt3YDTWOhegS++iCY/QBDTGAb3wEEqY
        HwfwRRzFX7MmfhoRDfkRkyIBD7zQBbcGHwdABbfAfRZBDO53HgbgBs5gEbXQAg8w
        OAemYB5gBoXABTpgAhfwAHCAlQzBDqJwgtpBBL9HEOSQA0Kwl3wpBD2QQATBDnrZ
        l3zZbxgRhdrBPywBlaZACG4QBmFABm+gCKgADOx3EdbQCoSQBpD5BpYgC2Z4Eazg
        BHVCaehTASj1ADdAB1hTEd+ACoRABmHgBpZQbjuBcYIxDpQABTegAiogAiOgAjfw
        A2sQlF7xhnN0GPcAD+zQDtGhD2ERAvGRAq3BDcuwDPOQEtj+cJ3ZKZT4EYcvgQl4
        0AcgQQ9zgAenkBDnmZ4MMQp4oAf4IBPvsAp6gAf2iQeGMAwDcZ8EsQr26QcEcQr2
        +Q4D8Qr2iQkHgQ8Gep94oAnc4BAiqB2GFxOaoAeHABLzMJ6vkBB+oAfsuRCn0AeH
        EJ8wYQ7j2Qd/oAd60AcoqgsCEaJ4IA4DYQgsigfZMBCaIKIkig+DwKJ9QKIByqIp
        uqJ9oAcEqhD3cGjfhxUZ2gcbihEhOqIwgQ9/wKKYMA3+8A6bwKJ6cKPPkKL66Q/q
        cKJ94KICsaIIKhDi4KN/EAsFMQ3jeQg3ig6YwKKaYCKh2Qq76JQhAQ7GIKMCYQ7G
        sA7+DDEPy9AMR2oQ2NAMBSGojEoQTbqh77AMgCoQQEoQ+PAMh1oQUXqpCboMN2oQ
        72AMy4ANCTGqoZoQv8Ciq1AQulCfruAP88CqArELRNoH7Amnf2Cm/vAJLPoIKFoP
        BDEMVUqoA3EIeDAIJEpOBnAAU3keKTcS+IAHfqAMofCeeDAK73Ct9RkKJCqe5CkQ
        28qgj3Ck7okJwmCfd+oPzyCe9/mksjqeq+CfBwoOA7GeliqgDOqmL6qjBjGteLAL
        tnCfhmCq4qoJ2DqezyAQ8eAH1Sqg9WkIxnoQNPoH6lAQ9PAMgIoPh9AHjyAQodAH
        mBCygyAQtvAHeCCs/rAOGjr+DCxKDQSxDCsaCuYwEOLQDd3QPxTCpxyBDyw6CIYQ
        CjSKokFbp3+wCwIBrI4wEEb7CSHbB4bwon9wCHrwB7vqD9lQpUELrH+Qpk1qoaHg
        q1C7n33Qqv4wCyjqtDSKB4/aqf+6oofwB6HwtD8qEDR6CHNbowLRpIZQtHaKENOK
        ogxRpyXrD4bwB6OADfUpo8AatQLhsn+QDSbaB59QED6Kn7bQDSo7EKZkIJMlEj47
        tv4gDuOpBzX7DizKnkorELpQpdIgENmAoox6ClXaB7YQCjdapYXrDyf7B8UQr30Q
        owIxCymKtP4gr/4QuGbqs38Qq/7gtgWBDyuqB7/rD8X+ULVuSg1VCqjFgLJY2qRS
        6g816qkCAQ7jWbkDkQvqq76/YLIoiw4ZigdYOgh6kAv4QKPo6w91igfx6at4ULMD
        wQ0+6qN6YAsEoY/OZRKh+6Hj6bj0wKKjkLR9sLT+MAr+mrzmsA7xSbt60L6wW6WP
        KhA+Crz5K74iKxDIa6kkug4s+qTQi6kr6rgC4QgsaqmbawzeC7yhMBAw2p0FYb4e
        SxB4YLVWm6akqwe6sAspKhB1+rEo+qTZMJ7OG7u2WxDvsKpDLKQRrH0GwgCXuRHM
        67zHi6uW2qNbvLr6C7Xk+7wom6jLgKIPOhBPWw9Nyq9MrMZjbLb+gA22MAqGcKL+
        LnzBMNwHOyzHKEqiz/AKIbui8huvfwCvqxC82bmg9lm5pUsQoTAKT5um/tCj7hLE
        /rAKf+AIWdvIocyi05AN2UANLCrDjdoMosyiQIqYw/PFYIyiYjye7MmjfXDGE3zH
        4cupKBuzcIzJLEoPkUoQdeq4KTywVYoJYuu8LzwQ0kvIBGHBf4AP+LAIVSuywNrI
        GdqmAxHJeJCdsVC1VrvDFXuxmPrMONoHPVrF/qCr5HysNXqfNbqw/kANwwCvAhEL
        VguzAkGS2kEAKRB5JRHGZLvLZizBFByy4YsPukANGzzMA5G1fRDC/tCx5NmknLzR
        J5zHWDueW8zL0izI1Az+t8osuLrAor4gEN2Qw+Fsx+ScndSwCzi9C8vgD6tKxgRB
        D+4sEK9wuQLtD7e6rsJwqwP8t73KoonKuwFNEK3QBSWAJgOAACngBnp4EgqNwj7N
        y75MwSfbB/psDkXqorSLBwRRDw08ELb6B/r5tXH8qnrw0iK9DFWas/4ADix60sGc
        0kUaprq6oUPdB0fqCsH7vUNM05KMEPQwnn/wCQ+KD7bA0ZxsDETKv4bMogacxn9Q
        D/QQ2shcp33AD/7QuiIq0C7Lopt7E109xgzdyw5tqR2rB68QC0Ja0Wods1prC6Lc
        tXsbp4MQC6+gtWTbqtxworHgn7T6vChtqStqta/+8ArxHJ9K3AeacLulu9MzPc6N
        jRDdcKL3mdke7A+PzaKFzLpCurChm94D0bsGjA+kPd5F+qE4Ib14IMZ+kK2WirKF
        jLC7aw6YoKIpigmAKqAA6qp/PKTpTQ/7TQ3XmqKGUKn7zZ7ES+CqIJ5pKqB1G70z
        a6ApeghYmrRVqweD4AuDgAetGg/2Ca8I7sMHoQ6LbLV4cAijMLECkeJ4EKawe581
        Gwv2adeYep/xSQ+hQOApOghiPBXrMAy+gA4PIQ6/IAxrLBDr8AupihD1MAzFUOWA
        u6KVe8UjThDm8Av6zBHZ8Au7AMAn0Qy/8At6jRnVXMIw4hFzXuchAbDriueKfN7n
        fv7ngB7ogj7ohF7ohn7oiJ7oir7ojN7ojv7okB7pkj7plF7pln7pmJ7pmr7pnN7p
        nv7poB7qoj7qpF7qpn7qqJ7qqr7qrN7qrv7qsB7rsj7rtF7rtn7ruJ7rur7rvN7r
        vv7rwB7swj7sxF7sxn7syJ7syr7szN7szv7s0B7t0j7t1F7tSREQADs=
        """
        return photo
        
class StdoutToLabel(Tkinter.Label):
    """
    The special widget that listens to sys.stdout and updates its
    label accordingly. This widget also owns the progressbar it is
    placed on and updates it every time sys.stdout is called.
    Use stop() to stop retrieving from sys.stdout.
    Credit: http://stackoverflow.com/questions/2914603/
    """

    def __init__(self, widget, progressbar=None, width='default'):
        Tkinter.Label.__init__(self)
        self.defstdout = sys.stdout
        self.widget = widget
        self.progressbar = progressbar
        
        if width == 'default':
            try:
                self.width = widget.cget('width')
            except:
                self.width = None
        else:
            self.width = width

    def flush(self):
        """
        Frame sys.stdout's flush method.
        """
        self.defstdout.flush()

    def write(self, string, end=None):
        """
        Frame sys.stdout's write method. Puts input
        strings in the widget.
        """
        self.defstdout.write(string)
        if string == "\n":
            pass
        elif (len(string) > 100 and
        self.width is not None and 
        self.width > 0):
            if ("Fatal error" in string):
                self.widget.config(text=
                "Fatal error: See log file for details")
            else:
                self.widget.config(text="See log file for details")
        elif string is not None:

            last_char = string[-1]

            if last_char != '\n' and string.startswith('\r'):
                number = string.lstrip("\r").rstrip("%")
                number = number.split(".")[0]
                number = int(number)
                self.progressbar["value"] = number                
                
            elif last_char != '\n':
                self.widget.config(text=string)

            elif last_char == '\n' and string.startswith('\r'):
                number = string.lstrip("\r").rstrip("%")
                number = number.split(".")[0]
                number = int(number)
                self.progressbar["value"] = number
                
            else:
                self.widget.config(text=string[:-1])
                
        self.update()
        self.progressbar.update()

    def start(self):
        """
        Starts retrieving.
        """
        sys.stdout = self

    def stop(self):
        """
        Stops retrieving.
        """
        sys.stdout = self.defstdout

    @property
    def errors(self):
        return self.defstdout.errors

    @property
    def encoding(self):
        return self.defstdout.encoding
        
def main():
    root = ScoaryGUI(None)
    root.title("Scoary")
    root.geometry("800x400")
    root.resizable(0,0)
    root.mainloop()
    
if __name__ == "__main__":
    pass



import sys
import logging
import os
import errno

# Note: The Matrix and QuadTree implementations are heavily based on 
# original implementations by Christian Storm Pedersen.

# Python 2/3 annoyances
try:
    xrange
except NameError:
    xrange = range

class PimpedFileHandler(logging.FileHandler):
	"""
	A filehandler that can create directories if needed.
	"""
	def __init__(self, filename, mode='w', encoding=None, delay=0):
		"""
		Initiates the filehandler and create dir if it does not exist.
		"""
		self.makedir(filename)
		logging.FileHandler.__init__(self, filename, mode, encoding, delay)
	
	def makedir(self, filename):
		path = os.path.dirname(filename)
		# Check if dir exists. If not, try to create it. Else, do nothing:
		if not os.path.isdir(path):
			try:
				os.makedirs(path)
			except OSError as e:
				sys.exit("CRITICAL: Need write permission to outdir")
		else: # Dir exists, but might not have permission
			if not (os.access(path, os.W_OK) and 
            os.access(path, os.X_OK)):
				sys.exit("CRITICAL: Need write permission to outdir")

def counted(fn):
    def wrapper(*args, **kwargs):
        wrapper.called += 1
        return fn(*args, **kwargs)
    wrapper.called = 0
    wrapper.__name__ = fn.__name__
    return wrapper
    
class ScoaryLogger(logging.Logger):

    def __init__(self, name, level=logging.DEBUG):
        super(ScoaryLogger, self).__init__(name, level)

    @counted
    def info(self, *args, **kwargs):
        super(ScoaryLogger, self).info(*args, **kwargs)

    @counted
    def warning(self, *args, **kwargs):
        super(ScoaryLogger, self).warning(*args, **kwargs)

    @counted
    def critical(self, *args, **kwargs):
        super(ScoaryLogger, self).warning(*args, **kwargs)

    @counted
    def error(self, *args, **kwargs):
        super(ScoaryLogger, self).warning(*args, **kwargs)
        
class Matrix:
    """
    A matrix stored as a list of lists
    """
    def __init__(self, dim, elm=sys.maxsize):
        """
        Builds empty nxn matrix
        """
        self.undef = sys.maxsize
        self.dim = int(dim)
        self.data = []
        for i in xrange(self.dim):
            self.data.append(self.dim * [elm])

    def __getitem__(self, i):
        """
        Get row i from Matrix
        """
        return self.data[i]

    def __str__(self):
        s = ""
        for i in xrange(self.dim):
            for j in xrange(self.dim):
                s += str(self.data[i][j]) + " "
            s += "\n"
        return s.strip("\n")


class QuadTree:
    """
    A basic QuadTree with names
    """
    def __init__(self, dim, names=None):
        """
        Constructs a quad tree of dimension dim and fills it with 0's
        """
        self.undef = sys.maxsize
        self.dim = dim
        n = self.dim + self.dim % 2
        if names is None:
            names = [str(x) for x in xrange(dim)]
        self.names = names
        self.level = []
        while n > 1:
            n += (n % 2)
            self.level.append(Matrix(n))
            n = (n+1) // 2

    def get_elm(self, i, j):
        """
        Returns the element at position (i,j) in the quad tree
        """
        return self.level[0][i][j]

    def insert_row(self, i, row):
        """
        Inserts row (of dim elements) as row number i
        """
        curr_row = row
        for l in self.level:
            if len(curr_row) % 2 == 1:
                curr_row.append(self.undef)
            next_row = []
            for j in xrange(len(curr_row)):
                l[i][j] = curr_row[j]
                if j % 2 == 1:
                    next_row.append(self.quad_min(i, j, l))
            i //= 2
            curr_row = next_row

    def insert_col(self, j, col):
        """
        Inserts col (of dim elements) as col number j
        """
        curr_col = col
        for l in self.level:
            if len(curr_col) % 2 == 1:
                curr_col.append(self.undef)
            next_col = []
            for i in xrange(len(curr_col)):
                l[i][j] = curr_col[i]
                if i % 2 == 1:
                    next_col.append(self.quad_min(i, j, l))
            j //= 2
            curr_col = next_col

    def min(self):
        """
        Returns minimum element stored in tree
        """
        return self.quad_min(0, 0, self.level[-1])

    def argmin(self):
        """
        Returns coordinates of minimum element in tree
        """
        i = j = 0
        for l in reversed(self.level[1:]):
            i, j = self.quad_argmin(i, j, l)
            i *= 2
            j *= 2
        return self.quad_argmin(i, j, self.level[0])

    def quad_min_all(self, i, j, l):
        """
        Returns the minimum element stored in the quad (i,j) and its
        coordinates
        """
        # Need even numbers
        i = (i//2) * 2
        j = (j//2) * 2
        return min((l[i][j], i, j),
                   (l[i+1][j], i+1, j),
                   (l[i][j+1], i, j+1),
                   (l[i+1][j+1], i+1, j+1))

    def quad_min(self, i, j, l):
        """
        Returns the minimum element stored in the quad containing (i,j)
        """
        return self.quad_min_all(i, j, l)[0]

    def quad_argmin(self, i, j, l):
        """
        Returns the coordinates of the minimum element in the quad 
        containing (i,j)
        """
        return self.quad_min_all(i, j, l)[1:]


class PhyloTree:
    """
    A class that represents a binary tree. Phylotrees can be nested. 
    They can also contain tips at left, right or both nodes. Stores the 
    max number of paths under each of the following 5 conditions:
    1. Free path to AB.
    2. Free path to Ab.
    3. Free path to aB.
    4. Free path to ab.
    5. No free path.
    """
    def __init__(self, leftnode, rightnode, GTC):
        """
        Constructs a phylotree and links it to its left and right nodes
        """
        # First check if left and right are tips (type = str). Then we 
        # need to look up the gene-trait combination value at that node.
        # Also check if left and right nodes are PhyloTrees. If not, 
        # recursively create PhyloTrees.

        if type(leftnode) is str:
            self.leftnode = Tip(GTC[leftnode])
        elif isinstance(leftnode, PhyloTree):
            self.leftnode = leftnode
        else:
            self.leftnode = PhyloTree(leftnode=leftnode[0],
                                      rightnode=leftnode[1],
                                      GTC=GTC)

        if type(rightnode) is str:
            self.rightnode = Tip(GTC[rightnode])
        elif isinstance(rightnode, PhyloTree):
            self.rightnode = rightnode
        else:
            self.rightnode = PhyloTree(leftnode=rightnode[0],
                                       rightnode=rightnode[1],
                                       GTC=GTC)

        # Initialize the max number of paths. Set to -1 meaning they 
        # cannot be reached
        self.maxvalues = \
            {"AB": -1, "Ab": -1, "aB": -1, "ab": -1, "0": -1}
        self.max_propairs = \
            {"AB": -1, "Ab": -1, "aB": -1, "ab": -1, "0": -1}
        self.max_antipairs = \
            {"AB": -1, "Ab": -1, "aB": -1, "ab": -1, "0": -1}
        self.calculate_max()
        self.max_contrasting_pairs = max(self.maxvalues.values())
        self.max_contrasting_propairs = max(self.max_propairs.values())
        self.max_contrasting_antipairs = max(
            self.max_antipairs.values())

    def calculate_max(self):
        """
        A method for calculating the max number of pairings under the 5
        conditions
        """
        for condition in ["AB", "Ab", "aB", "ab", "nofree"]:
            if condition in ["AB", "Ab", "aB", "ab"]:
                pairings = self.calculate_max_condition(condition)
                self.maxvalues[condition] = pairings["Total"]
                self.max_propairs[condition] = pairings["Pro"]
                self.max_antipairs[condition] = pairings["Anti"]
            else:  # Condition == nofree
                pairings = self.calculate_max_nofree()
                self.maxvalues["0"] = pairings["Total"]
                self.max_propairs["0"] = pairings["Pro"]
                self.max_antipairs["0"] = pairings["Anti"]

    def calculate_max_condition(self, condition):
        """
        When passed for example 'AB', finds out the 9 distinct 
        conditions and calculates the max. Here, the possibilities are
        illustrated when passed 'AB' (eg. a free path to AB exists):
        
        Left        Right
        AB          No free
        AB          Ab
        AB          aB
        AB          ab
        AB          AB
        No free     AB
        Ab          AB
        aB          AB
        ab          AB
        
        """
        Possible_conditions = set(["AB", "Ab", "aB", "ab"])
        Possible_conditions.remove(condition)
        # Now we have a list of the elements that are NOT condition:
        Otherconditions = list(Possible_conditions)  
        max_pairs_1 = -1
        max_pairs_2 = -1
        max_pairs_3 = -1
        max_pairs_4 = -1
        max_pairs_5 = -1
        max_pairs_6 = -1
        max_pairs_7 = -1
        max_pairs_8 = -1
        max_pairs_9 = -1

        max_propairs_1 = -1
        max_propairs_2 = -1
        max_propairs_3 = -1
        max_propairs_4 = -1
        max_propairs_5 = -1
        max_propairs_6 = -1
        max_propairs_7 = -1
        max_propairs_8 = -1
        max_propairs_9 = -1

        max_antipairs_1 = -1
        max_antipairs_2 = -1
        max_antipairs_3 = -1
        max_antipairs_4 = -1
        max_antipairs_5 = -1
        max_antipairs_6 = -1
        max_antipairs_7 = -1
        max_antipairs_8 = -1
        max_antipairs_9 = -1

        if (self.leftnode.maxvalues[condition] > -1 and 
        self.rightnode.maxvalues["0"] > -1):
            max_pairs_1 = (self.leftnode.maxvalues[condition] + 
                self.rightnode.maxvalues["0"])
            max_propairs_1 = (self.leftnode.max_propairs[condition] + 
                self.rightnode.max_propairs["0"])
            max_antipairs_1 = (self.leftnode.max_antipairs[condition] + 
                self.rightnode.max_antipairs["0"])

        if (self.leftnode.maxvalues[condition] > -1 and 
        self.rightnode.maxvalues[Otherconditions[0]] > -1):
            max_pairs_2 = (self.leftnode.maxvalues[condition] + 
                self.rightnode.maxvalues[Otherconditions[0]])
            max_propairs_2 = (self.leftnode.max_propairs[condition] + 
                self.rightnode.max_propairs[Otherconditions[0]])
            max_antipairs_2 = (self.leftnode.max_antipairs[condition] + 
                self.rightnode.max_antipairs[Otherconditions[0]])

        if (self.leftnode.maxvalues[condition] > -1 and 
        self.rightnode.maxvalues[Otherconditions[1]] > -1):
            max_pairs_3 = (self.leftnode.maxvalues[condition] + 
                self.rightnode.maxvalues[Otherconditions[1]])
            max_propairs_3 = (self.leftnode.max_propairs[condition] + 
                self.rightnode.max_propairs[Otherconditions[1]])
            max_antipairs_3 = (self.leftnode.max_antipairs[condition] + 
                self.rightnode.max_antipairs[Otherconditions[1]])

        if (self.leftnode.maxvalues[condition] > -1 and 
        self.rightnode.maxvalues[Otherconditions[2]] > -1):
            max_pairs_4 = (self.leftnode.maxvalues[condition] + 
                self.rightnode.maxvalues[Otherconditions[2]])
            max_propairs_4 = (self.leftnode.max_propairs[condition] + 
                self.rightnode.max_propairs[Otherconditions[2]])
            max_antipairs_4 = (self.leftnode.max_antipairs[condition] + 
                self.rightnode.max_antipairs[Otherconditions[2]])

        if (self.leftnode.maxvalues[condition] > -1 and 
        self.rightnode.maxvalues[condition] > -1):
            max_pairs_5 = (self.leftnode.maxvalues[condition] + 
                self.rightnode.maxvalues[condition])
            max_propairs_5 = (self.leftnode.max_propairs[condition] + 
                self.rightnode.max_propairs[condition])
            max_antipairs_5 = (self.leftnode.max_antipairs[condition] + 
                self.rightnode.max_antipairs[condition])

        if (self.leftnode.maxvalues["0"] > -1 and 
        self.rightnode.maxvalues[condition] > -1):
            max_pairs_6 = (self.leftnode.maxvalues["0"] + 
                self.rightnode.maxvalues[condition])
            max_propairs_6 = (self.leftnode.max_propairs["0"] + 
                self.rightnode.max_propairs[condition])
            max_antipairs_6 = (self.leftnode.max_antipairs["0"] + 
                self.rightnode.max_antipairs[condition])

        if (self.leftnode.maxvalues[Otherconditions[0]] > -1 and 
        self.rightnode.maxvalues[condition] > -1):
            max_pairs_7 = (self.leftnode.maxvalues[Otherconditions[0]] +
                self.rightnode.maxvalues[condition])
            max_propairs_7 = \
                (self.leftnode.max_propairs[Otherconditions[0]] + 
                self.rightnode.max_propairs[condition])
            max_antipairs_7 = \
                (self.leftnode.max_antipairs[Otherconditions[0]] + 
                self.rightnode.max_antipairs[condition])

        if (self.leftnode.maxvalues[Otherconditions[1]] > -1 and 
        self.rightnode.maxvalues[condition] > -1):
            max_pairs_8 = (self.leftnode.maxvalues[Otherconditions[1]] +
                self.rightnode.maxvalues[condition])
            max_propairs_8 = \
                (self.leftnode.max_propairs[Otherconditions[1]] + 
                self.rightnode.max_propairs[condition])
            max_antipairs_8 = \
                (self.leftnode.max_antipairs[Otherconditions[1]] + 
                self.rightnode.max_antipairs[condition])

        if (self.leftnode.maxvalues[Otherconditions[2]] > -1 and
        self.rightnode.maxvalues[condition] > -1):
            max_pairs_9 = (self.leftnode.maxvalues[Otherconditions[2]] +
                self.rightnode.maxvalues[condition])
            max_propairs_9 = \
                (self.leftnode.max_propairs[Otherconditions[2]] + 
                self.rightnode.max_propairs[condition])
            max_antipairs_9 = \
                (self.leftnode.max_antipairs[Otherconditions[2]] + 
                self.rightnode.max_antipairs[condition])

        max_pairs = max(max_pairs_1, max_pairs_2, max_pairs_3, 
                        max_pairs_4, max_pairs_5, max_pairs_6, 
                        max_pairs_7, max_pairs_8, max_pairs_9)

        # Calculate maximum number of propairs, given a maxmimum number 
        # of pairs
        max_propairs = -1
        if max_pairs == max_pairs_1:
            max_propairs = max(max_propairs, max_propairs_1)
        if max_pairs == max_pairs_2:
            max_propairs = max(max_propairs, max_propairs_2)
        if max_pairs == max_pairs_3:
            max_propairs = max(max_propairs, max_propairs_3)
        if max_pairs == max_pairs_4:
            max_propairs = max(max_propairs, max_propairs_4)
        if max_pairs == max_pairs_5:
            max_propairs = max(max_propairs, max_propairs_5)
        if max_pairs == max_pairs_6:
            max_propairs = max(max_propairs, max_propairs_6)
        if max_pairs == max_pairs_7:
            max_propairs = max(max_propairs, max_propairs_7)
        if max_pairs == max_pairs_8:
            max_propairs = max(max_propairs, max_propairs_8)
        if max_pairs == max_pairs_9:
            max_propairs = max(max_propairs, max_propairs_9)

        # Calculate maximum number of antipairs, given a maxmimum number
        # of pairs
        max_antipairs = -1
        if max_pairs == max_pairs_1:
            max_antipairs = max(max_antipairs, max_antipairs_1)
        if max_pairs == max_pairs_2:
            max_antipairs = max(max_antipairs, max_antipairs_2)
        if max_pairs == max_pairs_3:
            max_antipairs = max(max_antipairs, max_antipairs_3)
        if max_pairs == max_pairs_4:
            max_antipairs = max(max_antipairs, max_antipairs_4)
        if max_pairs == max_pairs_5:
            max_antipairs = max(max_antipairs, max_antipairs_5)
        if max_pairs == max_pairs_6:
            max_antipairs = max(max_antipairs, max_antipairs_6)
        if max_pairs == max_pairs_7:
            max_antipairs = max(max_antipairs, max_antipairs_7)
        if max_pairs == max_pairs_8:
            max_antipairs = max(max_antipairs, max_antipairs_8)
        if max_pairs == max_pairs_9:
            max_antipairs = max(max_antipairs, max_antipairs_9)

        return {"Total": max_pairs,
                "Pro": max_propairs,
                "Anti": max_antipairs}

    def calculate_max_nofree(self):
        """
        Under the condition of no free paths, only 5 distinct 
        possibilities exits. (No free paths requires either that there
        are no free paths in the left or right nodes, or that a new pair
        is formed across the root from the left and right nodes)
        
        Left        Right       Result
        No free     No free
        AB          ab          +1 pair (pro)
        ab          AB          +1 pair (pro)
        Ab          aB          +1 pair (anti)
        aB          Ab          +1 pair (anti)
        """
        # No free pairs in either:
        max_pairs_nofree = -1
        max_pairs_1100 = -1
        max_pairs_0011 = -1
        max_pairs_1001 = -1
        max_pairs_0110 = -1

        max_propairs_nofree = -1
        max_propairs_1100 = -1
        max_propairs_0011 = -1
        max_propairs_1001 = -1
        max_propairs_0110 = -1

        max_antipairs_nofree = -1
        max_antipairs_1100 = -1
        max_antipairs_0011 = -1
        max_antipairs_1001 = -1
        max_antipairs_0110 = -1

        if (self.leftnode.maxvalues["0"] > -1 and 
        self.rightnode.maxvalues["0"] > -1):
            max_pairs_nofree = (self.leftnode.maxvalues["0"] + 
                self.rightnode.maxvalues["0"])
            max_propairs_nofree = (self.leftnode.max_propairs["0"] + 
                self.rightnode.max_propairs["0"])
            max_antipairs_nofree = (self.leftnode.max_antipairs["0"] + 
                self.rightnode.max_antipairs["0"])

        if (self.leftnode.maxvalues["AB"] > -1 and 
        self.rightnode.maxvalues["ab"] > -1):
            max_pairs_1100 = (self.leftnode.maxvalues["AB"] + 
                self.rightnode.maxvalues["ab"] + 1)
            max_propairs_1100 = (self.leftnode.max_propairs["AB"] + 
                self.rightnode.max_propairs["ab"])
            max_antipairs_1100 = (self.leftnode.max_antipairs["AB"] + 
                self.rightnode.max_antipairs["ab"])
            max_propairs_1100 += 1

        if (self.leftnode.maxvalues["ab"] > -1 and 
        self.rightnode.maxvalues["AB"] > -1):
            max_pairs_0011 = (self.leftnode.maxvalues["ab"] + 
                self.rightnode.maxvalues["AB"] + 1)
            max_propairs_0011 = (self.leftnode.max_propairs["ab"] + 
                self.rightnode.max_propairs["AB"])
            max_antipairs_0011 = (self.leftnode.max_antipairs["ab"] + 
                self.rightnode.max_antipairs["AB"])
            max_propairs_0011 += 1

        if (self.leftnode.maxvalues["Ab"] > -1 and 
        self.rightnode.maxvalues["aB"] > -1):
            max_pairs_1001 = (self.leftnode.maxvalues["Ab"] + 
            self.rightnode.maxvalues["aB"] + 1)
            max_propairs_1001 = (self.leftnode.max_propairs["Ab"] + 
            self.rightnode.max_propairs["aB"])
            max_antipairs_1001 = (self.leftnode.max_antipairs["Ab"] + 
            self.rightnode.max_antipairs["aB"])
            max_antipairs_1001 += 1

        if (self.leftnode.maxvalues["aB"] > -1 and 
        self.rightnode.maxvalues["Ab"] > -1):
            max_pairs_0110 = (self.leftnode.maxvalues["aB"] + 
                self.rightnode.maxvalues["Ab"] + 1)
            max_propairs_0110 = (self.leftnode.max_propairs["aB"] + 
                self.rightnode.max_propairs["Ab"])
            max_antipairs_0110 = (self.leftnode.max_antipairs["aB"] + 
                self.rightnode.max_antipairs["Ab"])
            max_antipairs_0110 += 1

        max_pairs = max(max_pairs_nofree, max_pairs_1100,
                        max_pairs_0011, max_pairs_1001, max_pairs_0110)

        # Calculate max number of propairs
        max_propairs = -1  # Max_propairs can never go below -1
        if max_pairs == max_pairs_nofree:
            max_propairs = max(max_propairs, max_propairs_nofree)
        if max_pairs == max_pairs_1100:
            max_propairs = max(max_propairs, max_propairs_1100)
        if max_pairs == max_pairs_0011:
            max_propairs = max(max_propairs, max_propairs_0011)
        if max_pairs == max_pairs_1001:
            max_propairs = max(max_propairs, max_propairs_1001)
        if max_pairs == max_pairs_0110:
            max_propairs = max(max_propairs, max_propairs_0110)

        # Calculate max number of antipairs
        max_antipairs = -1  # Max_antipairs can never go below -1
        if max_pairs == max_pairs_nofree:
            max_antipairs = max(max_antipairs, max_antipairs_nofree)
        if max_pairs == max_pairs_1100:
            max_antipairs = max(max_antipairs, max_antipairs_1100)
        if max_pairs == max_pairs_0011:
            max_antipairs = max(max_antipairs, max_antipairs_0011)
        if max_pairs == max_pairs_1001:
            max_antipairs = max(max_antipairs, max_antipairs_1001)
        if max_pairs == max_pairs_0110:
            max_antipairs = max(max_antipairs, max_antipairs_0110)

        return {"Total": max_pairs,
                "Pro": max_propairs,
                "Anti": max_antipairs}


class Tip:
    """
    A class that references a single tip, which can only be AB, Ab, aB, 
    ab, A- or a-
    """
    def __init__(self, tipvalue):
        """
        Sets up the tip
        """
        self.tipvalue = tipvalue
        self.maxvalues = {}
        for condition in ["AB", "Ab", "aB", "ab", "0"]:
            if condition == tipvalue:
                self.maxvalues[condition] = 0
            else:
                self.maxvalues[condition] = -1
        self.max_propairs = {k: v for (k, v) in self.maxvalues.items()}
        self.max_antipairs = {k: v for (k, v) in self.maxvalues.items()}

if __name__ == '__main__':
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Script to search vcf files for mutations within specific coordinates
# Input: 
# -A vcf file
#
# Output:
# -A Roary-like file with mutations sorted in rows, strains as columns and presence/absence in cells
# -Columns: Chromosome, Position, variant (eg C->T), type (eg missense, synonymous, frameshift etc)


# Reading VCF
# File metainfo starts as ##key=value
# These are always formed and should be caught
# example ##fileformat=VCFv4.3 - give warning if format is off
# Columns 8 MANDATORY
# CHROM POS ID REF ALT QUAL FILTER INFO
# OPTIONAL COLUMNS
# FORMAT SAMPLE1 SAMPLE2 etc
# All data lines are tab-delimited
# CHROM : string, no whitespace
# POS : integer. Can have many lines with same pos. Pos=0 or N+1 for telomere positions
# ID : semicolon-delimited list of strings
# REF : string, ACGTN (can be multiple)
# ALT : comma-separated list, ACGTN* (* = allele is missing due to overlapping deletion)
# (NOTE: Suggest splitting ALT variants into different lines to preserve binarity)
# QUAL : float
# FILTER : PASS or semicolon-delimited list
# INFO : semicolon-delimited list of key=value pairs or flags
# FORMAT (optional) : colon-delimited list.
# Genotype fields - Genotype always first field
# GT encoded as allele values separated by | or /. 0 = reference. 1 = first ALT. 2 = second alt etc
# NOTE: Haploid calls (bacteria) have only 1 value
# NOTE: / means genotype unphased. | means genotype phased
# INFO field SVtypes : DELetion, INSertion, DUPlication, INVersion, CNV

import sys
import argparse
import os
import csv
import re
import traceback

__version__ = '0.1b'
__author__ = 'Ola Brynildsrud'
__credits = ['Ola Brynildsrud']
__email__ = 'olbb@fhi.no'

def main():
    """
    Converts VCF files (version 4.x) to Scoary format
    """
    ##########################################################################
    # Parse command line arguments

    parser = argparse.ArgumentParser(
        description='This script takes in vcf files and creates a '
                    'presence/absence matrix of mutations in the '
                    'Roary/Scoary format',
        epilog='by Ola Brynildsrud (olbb@fhi.no)')
    parser.add_argument(
        '--out',
        action='store',
        default='./mutations_presence_absence.csv',
        help='The path to the output file')
    parser.add_argument(
        '--types',
        action='store',
        default='ALL',
        help='The types of variants to include in the output. NOTE: This '
             'works if TYPE=XX can be found in the INFO column of the vcf '
             'file. The special keyword ALL includes all types. This is '
             'the default setting. Common types are snp, mnp, ins, del '
             'and complex. Give as comma-separated list. '
             'Example: --types snp,ins,del')
    parser.add_argument(
        '--version',
        action='version',
        version=__version__)
    parser.add_argument(
        '--force',
        action='store_true',
        default=False,
        help='Force overwriting of output file. (If it already '
             'exists)')
    parser.add_argument(
        'vcf',
        action='store',
        metavar='<VCF_file>',
        help='The VCF file to convert to Roary/Scoary format')

    args = parser.parse_args()
    if args.types is not "ALL":
        args.types = args.types.split(",")

    if os.path.isfile(args.out) and not args.force:
        sys.exit("Outfile already exists. Change name of outfile or "
                 "run with --force")
    if not os.path.isfile(args.vcf):
        sys.exit("Unable to locate input file %s" % args.vcf)

    with open(args.vcf,'rU') as vcffile, open(args.out,'w') as outfile:
        lines = csv.reader(vcffile, delimiter="\t")
        metainfo = {"##INFO" : {},
                    "##FILTER" : {},
                    "##FORMAT" : {},
                    "##ALT" : {},
                    "##contig" : {},
                    "##META" : {},
                    "##SAMPLE" : {},
                    "##PEDIGREE" : {}
        }
        #for line in lines:
        while True:
            try:
                line = next(lines)
            except StopIteration:
                print(traceback.print_exc())
                sys.exit("ERROR: There appears to be only metainformation "
                         "(lines starting with ##) in your VCF file.")
            # Get metainfo from file
            if line[0][:2] == '##':
                infoline = re.split('=',line[0], maxsplit=1)
                # Capture list output for complex tags
                if infoline[0] in metainfo:
                    ID=re.search(r'ID=(\w+)',infoline[1]).group(1)
                    infolist = re.split(',',infoline[1].strip("<>"))
                    metainfo[infoline[0]][ID] = {}
                    # Enter all elements in infolist into appropriate dic
                    for e in infolist:
                        esplit = e.split("=")
                        metainfo[infoline[0]][ID][esplit[0]] = esplit[1]

                else:
                    metainfo[infoline[0]] = infoline[1]
            else:
                # Have reached the data section of the file
                data = {"header": line}
                break

        try:
            vcfversion = metainfo["##fileformat"].split("v")[1]
            if int(vcfversion[0]) != 4:
                print("WARNING: A VCF format other than 4.x detected."
                      " File parsing may proceed with errors.")
            else:
                print("VCF version %s detected" % vcfversion)
        except:
            print("WARNING: Could not detect VCF format. Expected "
                  "v4.x. File parsing may proceed with errors.")
            print(traceback.print_exc())

        # Check that genotype fields have a single allele
        if metainfo["##FORMAT"]["GT"]["Number"] != "1":
            sys.exit("ERROR: Expected a single allele per genotype. Scoary "
                     "only works for haploid organisms.")

        # Have now caught all metainformation. Now get column information       
        #header = next(line)
        #print header
        data["header"] = data["header"][:9] + ["DUMMY"] + data["header"][9:]
        outfile.write(','.join('"' + c + '"' for c in data["header"]) + '\n')

        while True:
            try:
                line = next(lines)
            except StopIteration:
                print("Reached the end of the file")
                sys.exit(0)
            # Check if line is allowed:
            if args.types is not "ALL":
                vartype = re.search(r'TYPE=(\w+)',line[7]).group(1)
                if vartype not in args.types:
                    continue
            
            # Split line if ALT contains more than one variant
            if "," in line[4]:
                orgline = line[:]
                alts = line[4].split(",")
                c = 1
                for a in alts:
                     newline = orgline[:]
                     newline[4] = a
                     # Only get GT
                     newline[9:] = \
                         [cell.split(":")[0] for cell in orgline[9:]]
                     # Fix dummy comparisons
                     newline[9:] = fixdummy(newline[9:], c)
                     newline = newline[:9] + ["True"] + newline[9:]
                     c += 1
                     writeLine(newline, outfile)

            # Genotype fields need to be 0 or 1
            # GT is always first in colon-separated list
            else:
                newline = line[:9] + ["False"] + line[9:]
                writeLine(newline, outfile)

def writeLine(line, outfile):
    writeline = line[:9] + [cell.split(":")[0] for cell in line[9:]]
    outfile.write(','.join('"' + c + '"' for c in writeline) + '\n')

def fixdummy(line,c):
    newline = line[:]
    for x in range(len(line)):
        if int(line[x]) == c:
            newline[x] = "1"
        else:
            newline[x] = "0"
    return newline

########
# MAIN #
########
if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Script for associating Roary output with phenotypic trait (+/-)
# Idea and implementation: Ola Brynildsrud (olbb@fhi.no)

# Null hypothesis: Gene is equally distributed in Trait+ and Trait- 
# isolates

import argparse
import sys
import csv
import time
import random
import logging
import re
from multiprocessing import Pool, Value
from scipy import stats as ss
from scipy import spatial
from .citation import citation
from .classes import Matrix
from .classes import QuadTree
from .classes import PhyloTree
from .classes import PimpedFileHandler
from .classes import ScoaryLogger
from .__init__ import __version__
#import scoary

import os
from pkg_resources import resource_string, resource_filename

#SCOARY_VERSION = scoary.__version__
SCOARY_VERSION = __version__

# Python 2/3 annoyances
try:
    xrange
except NameError:
    xrange = range

# Set up log and message flow
log = ScoaryLogger(logging.getLogger('scoary'))
log.setLevel(logging.DEBUG)

logformat = '%(asctime)s    %(message)s'
logdatefmt='%m/%d/%Y %I:%M:%S %p'
formatter = logging.Formatter(fmt=logformat,datefmt=logdatefmt)
   
def main(**kwargs):
    """
    The main function of Scoary.
    """
        
    # If main has been ran from the GUI, then args already exists
    if len(kwargs) == 0:
        args, cutoffs = ScoaryArgumentParser()
    else:
        args = kwargs["args"]
        cutoffs = kwargs["cutoffs"]
        sys.stdout = kwargs["statusbar"]

    # If the citation arg has been passed, nothing should be done except
    # a call to citation
    if args.citation:
        sys.exit(citation())
   
    # If the test argument was used, all settings are overrided and 
    # defaulted
    if args.test:
        args.correction = ['I','EPW']
        args.delimiter = ','
        args.genes = os.path.join(
            resource_filename(__name__, 'exampledata'), 
            'Gene_presence_absence.csv')
        args.grabcols = []
        args.max_hits = None
        args.newicktree = None
        args.no_pairwise = False
        args.outdir = './'
        args.permute = 0
        args.p_value_cutoff = [0.05,0.05]
        args.restrict_to = None
        args.start_col = 15
        args.threads = 4
        args.traits = os.path.join(
            resource_filename(__name__, 'exampledata'),
            'Tetracycline_resistance.csv')
        args.upgma_tree = True
        args.write_reduced = False
        args.no_time = False
        args.collapse = False
        cutoffs = {"I": 0.05, "EPW": 0.05}
    
    # Start clock
    starttime = time.time()
    # currenttime is ONLY used to append to filenames, therefore, it
    # is set empty if args.no_time is True
    if args.no_time:
        currenttime = ""
    else:
        currenttime = time.strftime("_%d_%m_%Y_%H%M")

    # Outdir should end with slash    
    if not args.outdir.endswith("/"):
        args.outdir += "/"
    
    # Initiate console from sys.stdout (which might be a statusbar)
    console = logging.StreamHandler(sys.stdout)
    console.setFormatter(logging.Formatter('%(message)s'))
    console.setLevel(logging.INFO)
    log.addHandler(console)
    
    # Create log file
    log_filename = os.path.join(args.outdir,
                                "scoary%s.log" % currenttime)
    log_handler = PimpedFileHandler(log_filename, mode='w')
    log_handler.setFormatter(formatter)
    log.addHandler(log_handler)
    log.info("==== Scoary started ====")
    log.info("Command: " + " ".join(sys.argv))
    
    # Catch all system exceptions from here on out
    try:

        # Perform tests to make sure all input is okay                        
        if args.traits is None or args.genes is None:
            sys.exit(("The following arguments are required: -t/--traits, "
            "-g/--genes"))
        if args.threads <= 0:
            sys.exit("Number of threads must be positive")
        if not os.path.isfile(args.traits):
            sys.exit("Could not find the traits file: %s" % args.traits)
        if not os.path.isfile(args.genes):
            sys.exit("Could not find the gene presence absence file: %s" 
            % args.genes)
        if ((args.newicktree is not None) and 
        (not os.path.isfile(args.newicktree))):
            sys.exit("Could not find the custom tree file: %s"
            % args.newicktree)
        if not all( [(p <= 1.0 and p > 0.0) for p in args.p_value_cutoff] ):
            sys.exit("P must be between 0.0 and 1.0 or exactly 1.0")
        if (len(args.delimiter) > 1):
            sys.exit("Delimiter must be a single character string. "
            "There is no support for tab.")
        if ((len(args.p_value_cutoff) != len(args.correction)) and
        (len(args.p_value_cutoff) != 1)):
            sys.exit("You can not use more p-value cutoffs than correction "
            "methods. Either provide a single p-value that will be applied "
            "to all correction methods, or provide exactly as many as the "
            "number of correction methods and in corresponding sequence. "
            "e.g. -c I EPW -p 0.1 0.05 will apply an individual p-value "
            "cutoff of 0.1 AND a pairwise comparisons p-value cutoff of "
            "0.05.")
        if "P" in cutoffs and args.permute == 0:
            sys.exit("Cannot use empirical p-values in filtration without "
            "performing permutations. Use '--permute X' where X is a "
            "number equal to or larger than 10")
        if args.permute < 10 and args.permute != 0:
            sys.exit("The absolute minimum number of permutations is 10 "
            "(or 0 to deactivate)")
        elif args.permute > 0:
            try:
                random.shuffle
            except:
                sys.exit("Unable to find random.shuffle. Can not proceed "
                "with permutations")
        if "P" in cutoffs:
            if cutoffs["P"] < (1.0/args.permute):
                sys.exit("Permutation cutoff too low for this number of "
                "permutations")
        if args.permute > 10000:
            log.info("Note: You have set Scoary to do a high number of "
            "permutations. This may take a while.")
        if args.no_pairwise:
            log.info("Performing no pairwise comparisons. Ignoring all "
            "tree related options (user tree, population aware-correction, "
            "permutations).")
            args.permute = 0
            args.newicktree = None
            for m in ["PW","EPW","P"]:
                cutoffs.pop(m,None)
            
        # Start analysis
        with open(args.genes, "rU") as genes, \
        open(args.traits, "rU") as traits:
    
            if args.restrict_to is not None:
                # Note: Despite being a dictionary, the values of
                # allowed_isolates are not currently used, only the keys
                allowed_isolates = {isolate : "all"
                                    for line in
                                    open(args.restrict_to,"rU")
                                    for isolate in line.rstrip().split(",")}
            else:
                # Despite the confusing name
                # this actually means all isolates are allowed
                # and included in the analysis
                allowed_isolates = None
                if args.write_reduced:
                    sys.exit("You cannot use the -w argument without "
                    "specifying a subset (-r)")

            log.info("Reading gene presence absence file")
            
            if args.grabcols == "ALL":
                args.grabcols = [-999]
            genedic_and_matrix = \
                Csv_to_dic_Roary(genes,
                args.delimiter,
                args.grabcols,
                startcol=int(args.start_col) - 1,
                allowed_isolates=allowed_isolates,
                writereducedset=args.write_reduced,
                time=currenttime,
                outdir=args.outdir) 
            genedic = genedic_and_matrix["Roarydic"]
            zeroonesmatrix = genedic_and_matrix["Zero_ones_matrix"]
            strains = genedic_and_matrix["Strains"]
            extracolstoprint = genedic_and_matrix["Extracols"]
            firstcolnames = genedic_and_matrix["Firstcolnames"]

            # Create or load tree (No need for tree if --no_pairwise)
            if (args.newicktree) is None and not (args.no_pairwise):
                log.info("Creating Hamming distance matrix based on gene "
                "presence/absence")
                TDM = CreateTriangularDistanceMatrix(zeroonesmatrix,
                                                     strains)
                QT = PopulateQuadTreeWithDistances(TDM)
                log.info("Building UPGMA tree from distance matrix")
                upgmatree = upgma(QT)
            elif (args.no_pairwise):
                # Performing --no_pairwise analysis
                log.info("Ignoring relatedness among input sample and "
                "performing only population structure-naive analysis.")
                upgmatree = None
            else:
                log.info("Reading custom tree file")
                from .nwkhandler import ReadTreeFromFile
                upgmatree, members = ReadTreeFromFile(args.newicktree)
                if (sorted(strains) != sorted(members)):
                    # There could be two reasons for this. Either (a) that
                    # the gene presence absence file and the tree file does
                    # not have the same isolates or (b) that the -r flag has
                    # been used.
                    if args.restrict_to is None:
                        sys.exit("CRITICAL: Please make sure that isolates in "
                        "your custom tree match those in your gene "
                        "presence absence file.")
                    else:    
                        # Verify that the all isolates in strains (from GPA
                        # file) are in members (from tree), and if so prune
                        # tree of extraneous isolates. Else, call an error
                        if (all(i in members for i in strains)):
                            # Prune tree of the extraneous isolates
                            log.info("Pruning phylogenetic tree to correspond "
                            "to set of included isolates")
                            Prune = [i for i in members if 
                                          i not in strains]
                            upgmatree = PruneForMissing(upgmatree, Prune)
                        else:
                            sys.exit("CRITICAL: Your provided tree file did "
                            "not contain all the isolates in your gene "
                            "presence absence file.")
            log.info("Reading traits file")
            traitsdic, Prunedic = Csv_to_dic(traits,
                                             args.delimiter,
                                             allowed_isolates,
                                             strains)
            
        log.info("Finished loading files into memory.\n\n")
        log.info("==== Performing statistics ====")
        filtopts = filtrationoptions(cutoffs, args.collapse)
        for line in filtopts:
            log.info(line)

        log.info("Tallying genes and performing statistical analyses")

        RES_and_GTC = Setup_results(genedic, traitsdic, args.collapse)
        RES = RES_and_GTC["Results"]
        GTC = RES_and_GTC["Gene_trait_combinations"]

        if args.upgma_tree:
            StoreUPGMAtreeToFile(upgmatree,
                                 args.outdir,
                                 time=currenttime)        
    
        StoreResults(RES,
                     args.max_hits,
                     cutoffs,
                     upgmatree,
                     GTC,
                     Prunedic,
                     args.outdir,
                     args.permute,
                     args.threads,
                     args.no_pairwise,
                     genedic,
                     extracolstoprint,
                     firstcolnames,
                     time=currenttime,
                     delimiter=args.delimiter)
        log.info("\n")
        log.info("==== Finished ====")
        log.info("Checked a total of %d genes for "
              "associations to %d trait(s). Total time used: %d "
              "seconds." % (len(genedic),
                            len(traitsdic),
                            int(time.time()-starttime)))
    
    except SystemExit as e:
        exc_type, exc_value, _ = sys.exc_info()
        log.exception("CRITICAL:")
        log.removeHandler(log_handler)
        log.removeHandler(console)
        sys.exit(exc_value)
    
    if log.critical.called > 0:
        log.info("Scoary finished successfully, but with CRITICAL ERRORS. "
        "Please check your log file.")
    elif log.error.called > 0:
        log.info("Scoary finished successfully, but with ERRORS. Please check "
        "your log file.")
    elif log.warning.called > 0:
        log.info("Scoary finished successfully, but with WARNINGS. Please "
        "check your log file.")
    else:
        log.info("No warnings were recorded.")
    log.removeHandler(log_handler)
    log.removeHandler(console)
    sys.exit(0)

###############################
# FUNCTIONS FOR READING INPUT #
###############################
def Csv_to_dic_Roary(genefile, delimiter, grabcols, startcol=14, 
    allowed_isolates=None, writereducedset=False, time="",
    outdir="./"):
    """
    Converts a gene presence/absence file into dictionaries
    that are readable by Scoary.
    """
    r = {}

    if writereducedset:
        file = open(ReduceSet(genefile,delimiter, grabcols, startcol,
                    allowed_isolates,time,outdir),"rU")
        csvfile = csv.reader(file, skipinitialspace=True, 
                             delimiter=delimiter)
    else:
        csvfile = csv.reader(genefile, skipinitialspace=True, 
                             delimiter=delimiter)
    header = next(csvfile)
    
    # If include all relevant columns, grabcols == -999
    if grabcols == [-999]:
        grabcols = list(range(3,len(header)))

    try:
        header[startcol]
    except IndexError:
        sys.exit("The startcol (-s) you have specified does not seem to "
        "correspond to any column in your gene presence/absence file.")
    strains = header[startcol:]
    extracolstoprint = [header[c] for c in grabcols]
    
    # Include alternative column spellings
    Roarycols = ["Gene", "Non-unique Gene name", "Annotation",
                 "No. isolates", "No. sequences",
                 "Avg sequences per isolate", "Genome Fragment",
                 "Order within Fragment", "Accessory Fragment",
                 "Accessory Order with Fragment", "QC",
                 "Min group size nuc", "Max group size nuc",
                 "Avg group size nuc", "Order within fragment",
                 "Genome fragment", "Accessory fragment" ]
    
    if header[0:3] == Roarycols[0:3]:
        roaryfile = True
    else:
        roaryfile = False

    # Move forwards from startcol to find correct startcol
    if strains[0] in Roarycols and roaryfile:
        for c in xrange(len(strains)):
            if strains[c] not in Roarycols:
                correctstartcol = startcol + c
                break
        log.error("ERROR: Make sure you have set the -s parameter "
        "correctly. You are running with -s %s. This correponds to the "
        "column %s. If this is not an isolate, Scoary might crash or "
        "produce strange results. Scoary thinks you should have run "
        "with -s %s instead" % (str(startcol+1),
                                strains[0],
                                str(correctstartcol + 1)))
    
    if roaryfile:
        # Move backwards from startcol to find correct startcol
        Firstcols = header[:startcol][::-1]
        minus = 0
        Censored_isolates = []
        for c in xrange(len(Firstcols)):
            if Firstcols[c] not in Roarycols:
                minus += 1
                Censored_isolates.append(Firstcols[c])
            else:
                if minus > 0:
                    correctstartcol = startcol - minus
                    log.error("ERROR: Make sure you have set the -s "
                    "parameter correctly. You are running with -s %s. "
                    "Scoary thinks you should have used %s. This excludes "
                    "the following, which Scoary thinks are isolates: %s" % 
                    (str(startcol+1),
                    str(correctstartcol+1), 
                    ", ".join(Censored_isolates)))
                    break
    
    if allowed_isolates is not None:
        strain_names_allowed = [val for val in strains
                                if val in allowed_isolates.keys()]
    else:
        strain_names_allowed = strains

    zero_ones_matrix = []

    if roaryfile:
        try:
            genecol = header.index("Gene")
            nugcol = header.index("Non-unique Gene name")
            anncol = header.index("Annotation")
            firstcolnames = ["Gene", "Non-unique Gene name", "Annotation"]
        except ValueError:
            log.error("ERROR: Could not properly detect the correct names "
            "for all columns in the ROARY table.")
            genecol = 0
            nugcol = 1
            anncol = 2
    else:
        genecol = 0
        geneann = header[0]
        nugcol = 1
        nugann = header[1]
        anncol = 2
        annann = header[2]
        firstcolnames = header[0:3]

    for line in csvfile:
        q = line
        if roaryfile:
            try:
                identifier = q[genecol]
                r[identifier] = {"Non-unique Gene name": q[nugcol], 
                                  "Annotation": q[anncol]}
            except IndexError:
                sys.exit("CRITICAL: Could not read gene presence absence "
                "file. Verify that this file is a proper Roary file using "
                "the specified delimiter (default is ',').")
        else:
            try:
                identifier = ( q[genecol] + "_|_" 
                             + q[nugcol] + "_|_" 
                             + q[anncol] )
                             
                r[identifier] = {"Non-unique Gene name": q[nugcol], 
                                  "Annotation": q[anncol]}
            except IndexError:
                sys.exit("CRITICAL: Could not properly assign column. "
                         "Please report this bug.")
        # The zero_ones_line variable represents the presence (1) or 
        # absence (0) of a gene. It is used for calculating distances  
        # between strains.
        zero_ones_line = []

        for strain in xrange(len(strains)):
            if (allowed_isolates is not None):
                if strains[strain] not in allowed_isolates.keys():
                    continue
            if q[startcol + strain] in ["", "0", "-"]:
                # If the gene is not present, AND The isolate is allowed
                r[identifier][strains[strain]] = 0
                zero_ones_line.append(0)
                # Add a 0 to indicate non-presence
            else:
                # Gene is present if any other value than "", "0" or "-"
                # is in the cell
                r[identifier][strains[strain]] = 1
                zero_ones_line.append(1)
                # Add a 1 to indicate presence of the current gene in 
                # this strain

            # Add grabcols
            for c in grabcols:
                r[identifier][header[c]+"_name"] = q[c]

        # Since we are only interested in the differences between 
        # strains, no need to append the zero_ones_line if it is all 1's
        # (core gene) or all 0's (not in collection)
        if 1 in zero_ones_line and 0 in zero_ones_line:
            zero_ones_matrix.append(zero_ones_line)

    # Transpose list for distance calculation purposes
    if writereducedset:
        file.close()
    zero_ones_matrix = list(map(list, zip(*zero_ones_matrix)))

    return {"Roarydic": r,
            "Zero_ones_matrix": zero_ones_matrix,
            "Strains": strain_names_allowed,
            "Extracols": extracolstoprint,
            "Firstcolnames": firstcolnames}

def ReduceSet(genefile, delimiter, grabcols, startcol=14,
              allowed_isolates=None, time="",outdir="./"):
    """
    Helper function for csv_to_dic_roary.
    Method for writing a reduced gene presence absence file, based only
    on isolates allowed by the restrict_to flag. This can speed up
    Scoary when analyzing subsets of large (e.g. more than a couple 
    of hundred) datasets.
    """
    csvfile = csv.reader(genefile, skipinitialspace=True,
                         delimiter=delimiter)
    header = next(csvfile)
    allowed_indexes = list(range(startcol))
    for c in xrange(len(header)):
        if header[c] in allowed_isolates.keys():
            allowed_indexes.append(c)
    
    log.info("Writing gene presence absence file for the reduced set of "
          "isolates")

    reducedfilename = \
        "%sgene_presence_absence_reduced%s.csv" % (outdir, time)
    
    # Trim grabcols to only include non-restricted isolates
    grabcols = [i for i in grabcols if i in allowed_indexes]

    with open(reducedfilename, "w") as csvout:
        wtr = csv.writer(csvout, delimiter = delimiter)
        newheader = [header[a] for a in allowed_indexes]
        wtr.writerow(newheader)
        for r in csvfile:
            wtr.writerow( tuple(r[a] for a in allowed_indexes) )
    log.info("Finished writing reduced gene presence absence list to "
    "file %s" % str(reducedfilename))
    return reducedfilename

def Csv_to_dic(csvfile, delimiter, allowed_isolates, strains):
    """
    Converts an input traits file (csv format) to dictionaries readable 
    by Scoary.
    """
    tab = list(zip(*csv.reader(csvfile, delimiter=delimiter)))
    r = {}
    # Create dictionary over which trees need pruning due to missing 
    # data
    Prunedic = {}
    if len(tab) < 2:
        sys.exit("Please check that your traits file is formatted "
        "properly and contains at least one trait")
    for num_trait in xrange(1, len(tab)):
        p = dict(zip(tab[0], tab[num_trait]))
        if "" in p:
            name_trait = p[""]
            del p[""]
        elif "Name" in p:
            name_trait = p["Name"]
            del p["Name"]
        else:
            sys.exit("Make sure the top-left cell in the traits file "
            "is either empty or 'Name'. Do not include empty rows")

        # Filter so that only allowed isolates are included
        if allowed_isolates is not None:
            p = {strain: indicator for (strain, indicator) in 
            list(p.items()) if strain in allowed_isolates.keys()}
        # Stop if unknown character found in traits file
        allowed_values = ["0","1","NA",".","-"," ",""]
        if not all([x in allowed_values for x in p.values()]):
            sys.exit("Unrecognized character found in trait file. Allowed "
            "values (no commas): %s" % str(",".join(allowed_values)))
        # Remove isolates with missing values, but only for the 
        # trait for which they are missing
        if ("NA" in p.values() 
        or "-" in p.values() 
        or "." in p.values()
        or " " in p.values()
        or "" in p.values()):
            log.warning("WARNING: Some isolates have missing values for "
            "trait %s. Missing-value isolates will not be counted in "
            "association analysis towards this trait." 
            % str(name_trait))
            p_filt = {strain: indicator for (strain, indicator) in 
                p.items() if indicator not in ["NA","-","."," ",""]}
            Prunedic[name_trait] = [k for (k,v) in p.items() if
                v in ["NA","-","."," ",""]]
            #p = p_filt
        else:
            Prunedic[name_trait] = []
            p_filt = p
        # Remove isolates that did not have rows in the trait file but
        # that were allowed by the GPA file/restrict_to
        #if not all(s in p.keys() for s in strains):
        if not all(s in p.keys() for s in strains):
            log.error("ERROR: Some isolates in your gene presence "
            "absence file were not represented in your traits file. "
            "These will count as MISSING data and will not be included."
            )

            Prunedic[name_trait] += [s for s in strains if
                                     s not in p.keys() and
                                     s not in Prunedic[name_trait]]
        r[name_trait] = p_filt
        Prunedic[name_trait] += [None]

    return r, Prunedic

################################
# FUNCTIONS FOR HANDLING TREES #
################################
def CreateTriangularDistanceMatrix(zeroonesmatrix, strainnames):
    """
    Converts a raw matrix of 0s and 1s (indicating gene presence and 
    absence, rows correspond to genes and columns to allowed strains) to 
    an (upper) triangular matrix of pairwise Hamming distances.
    The distance d(i,i) is set to 1 for all i.
    """
    try:
        hamming_distances = [float(h) for h in list(
        spatial.distance.pdist(zeroonesmatrix, 'hamming'))]
    except TypeError:
        sys.exit("Could not locate scipy.spatial.distance.pdist. "
        "Perhaps you have an old version of SciPy installed?")
    nstrains = int((1 + (1 + 8*len(hamming_distances))**0.5)/2)
    TriangularDistanceMatrix = []
    Strain_names = []
    for i in xrange(nstrains):
        # Adding the maximum relative hamming distance to prevent 
        # Quadtree algorithm to pick a pair where i = j
        add = [1]  
        add += hamming_distances[:(nstrains-i-1)]
        hamming_distances = hamming_distances[(nstrains-i-1):]
        TriangularDistanceMatrix.append(add)
        Strain_names.append(strainnames[i])

    return {"matrix": TriangularDistanceMatrix, "names": Strain_names}

def PopulateQuadTreeWithDistances(TDM):
    """
    Takes a triangular distance matrix, creates a quadtree and populates
    it with the hamming distances between isolates.
    First creates a Quadmatrix, so not really optimized.
    """
    Quadmatrix = Matrix(dim=len(TDM["matrix"]))
    for i in xrange(Quadmatrix.dim):
        for j in xrange(i, Quadmatrix.dim):
            try:
                Quadmatrix[i][j] = \
                Quadmatrix[j][i] = TDM["matrix"][i][(j-i)]
            except IndexError:
                sys.exit("There was an error trying to populate the "
                         "Quadtree with pairwise distances. Please "
                         "report this bug to olbb@fhi.no")
    PopulatedQuadtree = QuadTree(Quadmatrix.dim, names=TDM["names"])
    for i in xrange(Quadmatrix.dim):
        PopulatedQuadtree.insert_row(i, Quadmatrix[i])
    return PopulatedQuadtree

def upgma(d):
    """
    Returns a UPGMA tree from a QuadTree distance matrix d. Heavily 
    based on original implementation by Christian Storm Pedersen.
    """
    n = d.dim
    cluster = [x for x in d.names]
    size = n * [1]

    while n > 1:
        # While there are more than a single cluster left, find clusters
        # i and j with minimum distance
        i, j = d.argmin()

        # Build list of distances from the new cluster to all the other
        # clusters
        new_cluster = [cluster[i], cluster[j]]
        new_size = size[i] + size[j]
        new_dist = []
        for k in xrange(d.dim):
            if cluster[k] is None:
                new_dist.append(1)
            else:
                new_dist.append((d.get_elm(i,k)*size[i] + 
                                 d.get_elm(j,k)*size[j]) / new_size)

        # Insert new row/col in d
        new_dist[i] = sys.maxsize
        d.insert_row(i, new_dist)
        d.insert_col(i, new_dist)
        d.insert_row(j, d.dim * [sys.maxsize])
        d.insert_col(j, d.dim * [sys.maxsize])

        cluster[i] = new_cluster
        cluster[j] = None
        size[i] = new_size
        size[j] = 0

        n -= 1

    return new_cluster

def PruneForMissing(tree, Prunedic):
    """
    Method for pruning isolates with missing values from the upgma tree
    and gene-trait combinations objects. This happens for a particular
    trait only, and so the full tree and GTC objects are used for traits
    without missing values
    """
    # Traverse tree and prune missing-data isolates
    # Left node is a subtree, go deeper

    # Make slice copy to prevent tampering with original
    tree = tree[:]
    
    if isinstance(tree[0],list):
        tree[0] = PruneForMissing(tree[0], Prunedic)
    # Right node is a subtree, go deeper
    if isinstance(tree[1], list):
        tree[1] = PruneForMissing(tree[1], Prunedic)
 
    # Both isolates have missing data and should be excised
    if (tree[0] in Prunedic) and (tree[1] in Prunedic):
        return
    # Only left node has missing data and should be excised
    elif tree[0] in Prunedic:
        return tree[1]
    # Only right node has missing data and should be excised
    elif tree[1] in Prunedic:
        return tree[0]
    # Neither the left or right nodes have missing data
    else:
        return [tree[0], tree[1]]

def StoreUPGMAtreeToFile(upgmatree, outdir, time=""):
    """
    A method for printing the UPGMA tree that is built internally from 
    the hamming distances in the gene presence/absence matrix
    """
    treefilename = str(outdir + ("Tree%s.nwk" % time))
    with open(treefilename, "w") as treefile:
        Tree = str(upgmatree)
        Tree = Tree.replace("[", "(")
        Tree = Tree.replace("]", ")")
        treefile.write(Tree + ";")
        log.info("Wrote the UPGMA tree to file: %s" % treefilename)       

#####################################
# FUNCTIONS FOR CALCULATING RESULTS #
#####################################
def Setup_results(genedic, traitsdic, collapse):
    """
    This is the method that actually does all the counting of genes,
    calculation of p-values and post-test adjustment of p-values.
    The majority of running time is currently spent doing Fisher's exact
    test. The running time is somewhat shortened by storing the p-values
    of known 2x2 cell distributions, so that identical Fisher's tests 
    will not have to be run.
    """
    # Need to create one results dictionary for each trait

    all_traits = {}
    gene_trait_combinations = {}
    fisher_calculated_values = {}
    for trait in traitsdic:
        all_traits[trait] = {}
        # Also need a list of all p-values to perform stepwise FDR corr
        p_value_list = []
        log.info("Gene-wise counting and Fisher's exact tests for trait: " 
              "%s" % str(trait))

        # We also need a number of tests variable for each genedic.
        # (The number of tests may not be the same, depending on how 
        # many genes are in 0 or all strains.)
        number_of_tests = len(genedic)
        # Need initial number of tests for status bar
        initial_number_of_tests = number_of_tests

        # Additionally, we need a dictionary to, for each gene
        # hold the gene-trait status (e.g "AB" or "ab" of each strain)
        gene_trait_combinations[trait] = {}
        distributions = {}
                
        i = 0  # Progress
        for gene in genedic:
            # Status:
            sys.stdout.write("\r{:.2%}".format(float(i) / 
                                               initial_number_of_tests))
            sys.stdout.flush()
            i += 1  # Progress

            stats = Perform_statistics(traitsdic[trait], genedic[gene])
            stat_table = stats["statistics"]

            num_pos = stat_table["tpgp"] + stat_table["tpgn"]
            num_neg = stat_table["tngp"] + stat_table["tngn"]

            if (stat_table["tpgp"] + stat_table["tngp"]) == 0:
                # No included isolates have the gene
                # Remove 1 from the number of tests
                number_of_tests -= 1  
                # proceed to the next gene
                continue  

            if (stat_table["tpgn"] + stat_table["tngn"]) == 0:
                # All included isolates have the gene
                number_of_tests -= 1
                continue

            # IF gene_trait_combination exists already, ie the current 
            # gene is perfectly correlated with another gene, collapse
            # the two into a single unit
            
            # Should consider adding a softer correlation required than 
            # 100%, i.e. genes that are highly co-distributed should
            #  perhaps be merged earlier?
            if stats["hash"] in distributions and collapse:
                # Find out which gene is correlated
                corr_gene = distributions[stats["hash"]]
                # Collapse the two genes
                newgenename = corr_gene + "--" + gene
                distributions[stats["hash"]] = newgenename
                # Remove 1 from the number of tests
                number_of_tests -= 1
                mergedgenes = True
                
                del(gene_trait_combinations[trait][corr_gene])
                gene_trait_combinations[trait][newgenename] = \
                                                     stats["gene_trait"]
            else:
                distributions[stats["hash"]] = gene
                mergedgenes = False
                gene_trait_combinations[trait][gene] = \
                                                     stats["gene_trait"]
  
            obs_table = [[stat_table["tpgp"],
                          stat_table["tpgn"]],
                         [stat_table["tngp"],
                          stat_table["tngn"]]]
            obs_tuple = (stat_table["tpgp"],
                         stat_table["tpgn"],
                         stat_table["tngp"],
                         stat_table["tngn"])
            if obs_tuple in fisher_calculated_values:
                odds_ratio = fisher_calculated_values[obs_tuple][0]
                p_value = fisher_calculated_values[obs_tuple][1]
            else:
                fisher = ss.fisher_exact(obs_table)
                p_value = fisher[1]
                odds_ratio = fisher[0]
                fisher_calculated_values[obs_tuple] = fisher
                 
            if not mergedgenes:
                all_traits[trait][gene] = {
                "NUGN": genedic[gene]["Non-unique Gene name"],
                "Annotation": genedic[gene]["Annotation"],
                "tpgp": stat_table["tpgp"],
                "tngp": stat_table["tngp"],
                "tpgn": stat_table["tpgn"],
                "tngn": stat_table["tngn"],
                "sens": ((float(stat_table["tpgp"]) / num_pos * 100) if 
                    num_pos > 0 else 0.0),
                "spes": ((float(stat_table["tngn"]) / num_neg * 100) if 
                    num_neg > 0 else 0.0),
                "OR": odds_ratio,
                "p_v": p_value}
                p_value_list.append((gene, p_value))
            else:
                temp = all_traits[trait][corr_gene]
                del(all_traits[trait][corr_gene])
                all_traits[trait][newgenename] = {
                "NUGN": temp["NUGN"] + "--" + 
                        genedic[gene]["Non-unique Gene name"],
                "Annotation": temp["Annotation"] + "--" + 
                              genedic[gene]["Annotation"],
                "tpgp": stat_table["tpgp"],
                "tngp": stat_table["tngp"],
                "tpgn": stat_table["tpgn"],
                "tngn": stat_table["tngn"],
                "sens": ((float(stat_table["tpgp"]) / num_pos * 100) if 
                    num_pos > 0 else 0.0),
                "spes": ((float(stat_table["tngn"]) / num_neg * 100) if
                    num_neg > 0 else 0.0),
                "OR": odds_ratio,
                "p_v": p_value}
                p_value_list.append((newgenename, p_value))
                
        
        sys.stdout.write("\r100.00%")
        sys.stdout.flush()
        sys.stdout.write("\n")
        log.info("Adding p-values adjusted for testing multiple hypotheses")
        
        # Now calculate Benjamini-Hochberg and Bonferroni p-values
        # Sorted list of tuples: (gene, p-value)
        # s_p_v = abbreviation for sorted_p_values
        s_p_v = sorted(p_value_list, key=lambda x: x[1])  

        # Find out which p-values are ties
        # Note: Changed from step-down to step-up
        # (from least significant to most significant)
        tie = [s_p_v[i-1][1] == s_p_v[i][1] for i in
            xrange(1, len(s_p_v))]
        # Initialize dics of corrected p values
        # bh_c_p_v = abbreviation for bh_corrected_p_values
        bh_c_p_v = {}
        # The least significant gene is entered into the dic
        bh_c_p_v[s_p_v[len(s_p_v)-1][0]] = last_bh = s_p_v[len(s_p_v)-1][1]

        for (ind, (gene, p)) in reversed(list(enumerate(s_p_v[:-1]))):
            bh_c_p_v[gene] = min([last_bh, p*number_of_tests/(ind+1.0)]\
                                ) if not tie[ind] else last_bh
            last_bh = bh_c_p_v[gene]

        # Now add values to dictionaries:
        for gene in all_traits[trait]:
            all_traits[trait][gene]["B_p"] = min(
                all_traits[trait][gene]["p_v"] * number_of_tests , 1.0)
            all_traits[trait][gene]["BH_p"] = min(bh_c_p_v[gene], 1.0)

    return {"Results": all_traits,
            "Gene_trait_combinations": gene_trait_combinations}

def Perform_statistics(traits, genes):
    """
    Helper function for Setup_results.
    The method that adds the presence/absence status for each 
    trait-status combination.
    """
    r = {"tpgp": 0, "tpgn": 0, "tngp": 0, "tngn": 0}
    # tpgn = trait positive, gene negative, etc
    gene_trait = {}
    distribution_hash = ""
    for t in traits:  # For each strain
        # If missing data, continue without including isolate
        # Note, this should no longer be invoked, since missing-value
        # isolates are not in the traits dic to begin with
        if traits[t] in ["NA","-","."]:
            gene_trait[t] = "NA"
            distribution_hash += str(genes[t])
            continue

        try:
            if int(traits[t]) == 1 and genes[t] == 1:
                r["tpgp"] += 1
                distribution_hash += "1"
                gene_trait[t] = "AB"
            elif int(traits[t]) == 1 and genes[t] == 0:
                r["tpgn"] += 1
                distribution_hash += "0"
                gene_trait[t] = "aB"
            elif int(traits[t]) == 0 and genes[t] == 1:
                r["tngp"] += 1
                distribution_hash += "1"
                gene_trait[t] = "Ab"
            elif int(traits[t]) == 0 and genes[t] == 0:
                r["tngn"] += 1
                distribution_hash += "0"
                gene_trait[t] = "ab"
            else:
                sys.exit("There was a problem with comparing your "
                "traits and gene presence/absence files. Make sure you "
                "have formatted the traits file to specification and "
                "only use 1s and 0s, as well as NA, - or . for missing "
                "data. Also make sure the Roary file contains empty "
                "cells for non-present genes and non-empty text cells "
                "for present genes.")
        except KeyError:
            sys.stdout.write("\n")
            log.critical("CRITICAL: Could not find %s in the genes "
            "file." % str(t) )
            #log.warning("CRITICAL: Could not find %s in the genes file" % str(t))
            sys.exit("Make sure strains are named the same in your "
            "traits file as in your gene presence/absence file")
    return {"statistics": r, "hash": int(distribution_hash, 2),
            "gene_trait": gene_trait}

#################################
# FUNCTIONS FOR CREATING OUTPUT #
#################################
def StoreResults(Results, max_hits, cutoffs, upgmatree, GTC, Prunedic,
                 outdir, permutations, num_threads, no_pairwise,
                 genedic, extracolstoprint, firstcolnames, time="", 
                 delimiter=","):
    """
    A method for storing the results. Calls StoreTraitResult for each 
    trait column in the input file.
    """
    for Trait in Results:
        sys.stdout.write("\n")
        log.info("Storing results: " + Trait)
        StoreTraitResult(Results[Trait], Trait, max_hits, cutoffs, 
                         upgmatree, GTC, Prunedic, outdir, permutations, 
                         num_threads, no_pairwise, genedic,
                         extracolstoprint, firstcolnames, time, delimiter)

def StoreTraitResult(Trait, Traitname, max_hits, cutoffs, upgmatree, 
                     GTC, Prunedic, outdir, permutations, num_threads, 
                     no_pairwise, genedic, extracolstoprint, 
                     firstcolnames, time="", delimiter=","):
    """
    The method that actually stores the results. Only accepts results 
    from a single trait at a time
    """
    permutations = int(permutations)
    if num_threads > 1:
        multithreaded = True
    else:
        multithreaded = False
    
    fname = (outdir + Traitname + time + '.results.csv')

    with open(fname, "w") as outfile:
        # Sort genes by p-value.
        sort_instructions = SortResultsAndSetKey(Trait)
        if max_hits is None:
            max_hits = len(Trait)
        num_results = min(max_hits, len(Trait))

        cut_possibilities = {
            "I": "p_v",
            "B": "B_p",
            "BH": "BH_p",
            "PW": "Plowest",
            "EPW": "Pboth",
            "P": "Empirical_p"
        }
        
        columns = firstcolnames + [
        "Number_pos_present_in","Number_neg_present_in",
        "Number_pos_not_present_in","Number_neg_not_present_in",
        "Sensitivity","Specificity","Odds_ratio","Naive_p",
        "Bonferroni_p",
        "Benjamini_H_p"]

        if not no_pairwise:
            columns += ["Max_Pairwise_comparisons",
                        "Max_supporting_pairs","Max_opposing_pairs",
                        "Best_pairwise_comp_p","Worst_pairwise_comp_p"]
        
        if permutations >= 10:
            columns.append("Empirical_p")

        columns += extracolstoprint
        
        outfile.write(delimiter.join('"' + c +'"' for c in columns) + 
                      "\n")
        
        if permutations >= 10:
            log.info("Calculating max number of contrasting pairs for "
            "each significant gene and performing %s permutations" 
            % str(permutations))
        elif no_pairwise:
            log.info("Skipping population structure-aware analyses.")
        else:
            log.info("Calculating max number of contrasting pairs for "
            "each nominally significant gene")
        

        if not no_pairwise:    
            # If some isolates have missing values, prune tree and GTC of 
            # these isolates
            if len(Prunedic[Traitname]) > 0:
                upgmatree = PruneForMissing(upgmatree, Prunedic[Traitname])
    
        # Use threads to resolve pairwise comparisons and permutations
        # Each subprocess will be responsible for a domain of the 
        # results list
        
            if multithreaded:
                domains = [list(xrange(tnum, num_results, num_threads)) 
                           for tnum in xrange(num_threads)]
                # The Progress variable holds the total progress status of 
                # the subprocesses. It is locked, but can be written to by 
                # all subprocs
                Progress = Value("h",0)
                pool = Pool(processes = num_threads,initializer=initProcess,
                            initargs=(Progress,))
                argumentlist = {"si":sort_instructions,"tree":upgmatree,
                                "GTC": GTC[Traitname],"cutoffs": cutoffs,
                                "cp": cut_possibilities,"perm":permutations,
                                "Trait":Trait, "Threaded": True}
                # Need to make a copy of dict to prevent then from pointing 
                # to the same object
                all_args = [ (domains[x], dict(argumentlist)) 
                             for x in xrange(len(domains)) ] 
        
                Threadresults = pool.imap(PairWiseComparisons,all_args)

                pool.close()
                pool.join()
            else:
                domains = list(xrange(num_results))
                argumentlist = {"si":sort_instructions,"tree":upgmatree,
                                "GTC": GTC[Traitname],"cutoffs": cutoffs,
                                "cp": cut_possibilities,"perm":permutations,
                                "Trait":Trait, "Threaded": False}
                all_args = (domains, argumentlist)
                Threadresults = [PairWiseComparisons(all_args)]
            sys.stdout.write("\r100.00%")
            sys.stdout.flush()
            sys.stdout.write("\n")

            # Wait for each thread to finish and weave results from all 
            # threads

            # Finally, create a filteredresults that merges results data 
            # from Trait[currentgene] with Threadresult[currentgene]
            Filteredresults = {}
            Threadresults = list(Threadresults)

            for thread in xrange(num_threads):
                for currentgene in Threadresults[thread]:
                    Filteredresults[currentgene] = \
                        Threadresults[thread][currentgene]
                    Filteredresults[currentgene].update(Trait[currentgene])
       
            # Only sort by pairwise values if they are the only present 
            # correction methods
            if ("I" in cutoffs) or ("B" in cutoffs) or ("BH" in cutoffs):
                sort_instructions = SortResultsAndSetKey(Filteredresults)
            elif ("PW" in cutoffs) or ("EPW" in cutoffs):
                sort_instructions = \
                    SortResultsAndSetKeyPairwise(Filteredresults,cutoffs)
            elif ("P" in cutoffs):
                sort_instructions = \
                    SortResultsAndSetKey(Filteredresults,key="Empirical_p")
            else:
                log.info("No filtration applied")
        
        # If no_pairwise mode was run - merge up after this clause.
        else:
            Filteredresults = Trait
            sort_instructions = SortResultsAndSetKey(Filteredresults)

        log.info("Storing results to file")       
        num_filteredresults = min(num_results, len(Filteredresults))
        for x in xrange(num_filteredresults):
            currentgene = sort_instructions[x]
            
            # Final check that the currentgene passes all filters
            if all( [Filteredresults[currentgene]\
                    [cut_possibilities[method]] <= cutoffs[method]
                    for method in cutoffs] ):
                # Split up identifier if it is a compound id
                if "_|_" in currentgene:
                    firstthree = currentgene.split("_|_")
                else:
                    firstthree = [currentgene,
                    str(Filteredresults[currentgene]["NUGN"]),
                    str(Filteredresults[currentgene]["Annotation"])]

                outrow = firstthree + [ 
                str(Filteredresults[currentgene]["tpgp"]),
                str(Filteredresults[currentgene]["tngp"]),
                str(Filteredresults[currentgene]["tpgn"]),
                str(Filteredresults[currentgene]["tngn"]),
                str(Filteredresults[currentgene]["sens"]),
                str(Filteredresults[currentgene]["spes"]),
                str(Filteredresults[currentgene]["OR"]),
                str(Filteredresults[currentgene]["p_v"]),
                str(Filteredresults[currentgene]["B_p"]),
                str(Filteredresults[currentgene]["BH_p"])]
                if not no_pairwise:
                    outrow += [
                    str(Filteredresults[currentgene]["max_total_pairs"]),
                    str(Filteredresults[currentgene]["max_propairs"]),
                    str(Filteredresults[currentgene]["max_antipairs"]),
                    str(Filteredresults[currentgene]["Pbest"]),
                    str(Filteredresults[currentgene]["Pworst"])]
            
                    # If permutations have been performed, print empirical 
                    # p-values as well
                    if permutations >= 10:
                        outrow.append(
                        str(Filteredresults[currentgene]["Empirical_p"]))
                        
                # Add extracols
                for col in extracolstoprint:
                    # Make sure to also get collapsed genes
                    if "--" in currentgene:
                        currentgenes = currentgene.split("--")
                        appendlist = [str(genedic[g][col+"_name"])
                                      for g in currentgenes]
                        appendstring = "--".join(appendlist)
                    else:
                        appendstring = str(genedic[currentgene][col+"_name"])
                    outrow.append(appendstring)
                outfile.write(
                delimiter.join('"' + c + '"' for c in outrow) + "\n"
                )

def initProcess(share):
    """
    Initiates the progressbar as a shared variable between multiple
    processes.
    """
    #scoary.Progress = share
    global Progress
    Progress = share

def PairWiseComparisons(nestedlist):
    """
    A threaded version of the pairwise comparisons correction method. 
    Also calculates permutations.
    """
    domain = nestedlist[0]
    sort_instructions = nestedlist[1]["si"]
    tree = nestedlist[1]["tree"]
    GTC = nestedlist[1]["GTC"]
    cutoffs = nestedlist[1]["cutoffs"]
    cutposs = nestedlist[1]["cp"]
    permutations = nestedlist[1]["perm"]
    Trait = nestedlist[1]["Trait"]
    Threaded = nestedlist[1]["Threaded"]
    
    resultscontainer = {}
    num_tot_res = len(Trait)
    progress = 0.0

    for genenumber in domain:
        if Threaded:
            #with scoary.Progress.get_lock():
            #    scoary.Progress.value += 1
            with Progress.get_lock():
                Progress.value += 1
            sys.stdout.write(
            #"\r{:.2%}".format(float(scoary.Progress.value)/num_tot_res)
            "\r{:.2%}".format(float(Progress.value)/num_tot_res)
            )
            sys.stdout.flush()
        else:
            progress += 1.0
            sys.stdout.write("\r{:.2%}".format(progress/num_tot_res))
            sys.stdout.flush()
        # Start with lowest p-value, the one which has key 0 in 
        # sort_instructions
        currentgene = sort_instructions[genenumber]
        resultscontainer[currentgene] = {}
        
        Max_pairwise_comparisons = ConvertUPGMAtoPhyloTree(
            tree, GTC[currentgene])

        resultscontainer[currentgene]["max_total_pairs"] = \
            Max_pairwise_comparisons["Total"]
        resultscontainer[currentgene]["max_propairs"] = \
            Max_pairwise_comparisons["Pro"]
        resultscontainer[currentgene]["max_antipairs"] = \
            Max_pairwise_comparisons["Anti"]
        
        # Switch definition of "best" and "worst" if antipairs outnumber
        # propairs
        if (Max_pairwise_comparisons["Pro"] >= 
            Max_pairwise_comparisons["Anti"]):
            Pbest = "Pbest"
            Pworst = "Pworst"
        else:
            Pbest = "Pworst"
            Pworst = "Pbest"
        try:
            resultscontainer[currentgene][Pbest] = ss.binom_test(
                resultscontainer[currentgene]["max_propairs"],
                resultscontainer[currentgene]["max_total_pairs"],
                0.5)
            resultscontainer[currentgene][Pworst] = ss.binom_test(
                resultscontainer[currentgene]["max_total_pairs"] - 
                resultscontainer[currentgene]["max_antipairs"],
                resultscontainer[currentgene]["max_total_pairs"],
                0.5)
            resultscontainer[currentgene]["Plowest"] = min(
                resultscontainer[currentgene]["Pbest"], 
                resultscontainer[currentgene]["Pworst"])
            resultscontainer[currentgene]["Pboth"] = max(
                resultscontainer[currentgene]["Pbest"],
                resultscontainer[currentgene]["Pworst"])
        except TypeError:
            sys.exit("There was a problem using " 
            "scipy.stats.binom_test. Ensure you have a recent "
            "distribution of SciPy installed.")

        # Loop can break early if filtration includes non-pairwise 
        # correction measures, because if these are all above the p
        # then we are not interested in the pairwise results
        
        resultscontainer[currentgene]["p_v"] = Trait[currentgene]["p_v"]
        resultscontainer[currentgene]["B_p"] = Trait[currentgene]["B_p"]
        resultscontainer[currentgene]["BH_p"] = Trait[currentgene]["BH_p"]
        
        if decideifbreak(cutoffs, resultscontainer[currentgene]):
            # Genes that are cut from the break
            # should not be in the results
            del(resultscontainer[currentgene])
            break    

        # This is also the place to add permutations - 
        # AFTER it has been verified that the current gene passes all 
        # filtration filters
        if permutations >= 10:
            empirical_p_perm = Permute(tree=tree,
                                       GTC=GTC[currentgene],
                                       permutations=permutations,
                                       cutoffs=cutoffs)
            resultscontainer[currentgene]["Empirical_p"] = \
                empirical_p_perm
    
    return resultscontainer
    
def Permute(tree, GTC, permutations, cutoffs):
    """
    A method used to permute the GTC dataset by label-switching a given 
    number of times and calculate corresponding pairwise comparisons
    p-values for each PhyloTree. Finally, it returns the percentile of 
    the unpermuted dataset p-value in the set of ordered p-values 
    resulting from the permutation.
    """
    proceed = True
    # Guard the original GTC against changes by making a copy
    GTCcopy = dict(GTC)
    # r is the number of replicates with a larger test statistic than 
    # the unpermuted
    r = 0
    Unpermuted_tree = ConvertUPGMAtoPhyloTree(tree,GTCcopy)
    
    # Find out if we're after propairs or antipairs - which is more 
    # common and would thus give the high test statistic?

    if Unpermuted_tree["Pro"] >= Unpermuted_tree["Anti"]:
        Pro = "Pro"
    else:
        Pro = "Anti"
    Unpermuted_estimator = (float(Unpermuted_tree[Pro]) / 
                           Unpermuted_tree["Total"])

    if permutations < 10:
        # The program is never supposed to go here since fewer than 10
        # permutations are globally disallowed
        sys.stdout.write("Number of permutations too few. The absolute "
        "minimum is 10.")
        proceed = False

    if proceed:
        for i in xrange(permutations):
            # Make a permutation using random.shuffle
            PermutedGTC = PermuteGTC(GTCcopy)
            # Send new set to phylotree
            NewPhyloTree = ConvertUPGMAtoPhyloTree(tree, PermutedGTC)
            if (float(NewPhyloTree[Pro]) / NewPhyloTree["Total"] >= 
            Unpermuted_estimator):
                r += 1

            # Check how many estimators are higher than the unpermuted
            # If, after more than 30 iterations, r indicates a p > 0.1,
            # abort
            if i >= 30:
                if (1 - ss.binom.cdf(r,i,0.1)) < 0.05:
                    emp_p = (r+1.0)/(i+2.0)
                    break
        else:
            emp_p = (r+1.0)/(permutations+1.0)
    else:
        return None
    
    return emp_p

def PermuteGTC(GTC):
    """
    Returns a permutation of the gene-trait combination dic where trait 
    labels have been swapped
    """
    # Set of trait labels to distribute
    trait_labels = [s[-1] for s in GTC.values()]
    # Shuffle list of labels
    random.shuffle(trait_labels)
    for isolate in GTC:
        # Assign the latter character of GTC to isolates but keep the 
        # gene status intact
        GTC[isolate] = str(GTC[isolate][0] + trait_labels.pop())
    return GTC

def ConvertUPGMAtoPhyloTree(tree, GTC):
    """
    A method that converts the upgma tree (in nested list form) to a 
    PhyloTree. It also needs the status (AB, Ab, aB or ab) of all
    strains that are to be included in the analysis. As of 1.6.0 this
    function is also called from permutations
    """

    # TRAVERSING TREE: For each binary division - go to left until hit 
    # tip. Then go back
    MyPhyloTree = PhyloTree(leftnode=tree[0],
                            rightnode=tree[1],
                            GTC=GTC)

    return {"Total": MyPhyloTree.max_contrasting_pairs,
            "Pro": MyPhyloTree.max_contrasting_propairs,
            "Anti": MyPhyloTree.max_contrasting_antipairs}

##############################
# CURRENTLY UNUSED FUNCTIONS #
##############################
def FindClosestIndex(sortedlist, value, index=None):
    """
    Takes a sorted list and a value and finds which index position the 
    value would have if entered into the list. Does this through a
    binary search
    """
    # METHOD DEPRECATED WHEN P-VALUE STATISTICS NOT NEEDED
    if index is None:
        index = 0
        
    listlen = len(sortedlist)
    if listlen == 0:
        return None
    if listlen == 1:
        if value <= sortedlist[0]:
            return 0
        else:
            return 1
    # Bisect list
    middleindex = listlen // 2
    middlevalue = sortedlist[middleindex]
    if value == middlevalue:
        # Ties in list. Place value exactly at index
        return middleindex
    elif value < middlevalue:
        # Index in left part of list
        index += FindClosestIndex(sortedlist[:middleindex],
                                  value,
                                  index)
    else:
        # Index in right part of list
        index += (middleindex + 
                  FindClosestIndex(sortedlist[middleindex:],
                                   value,
                                   index))
    
    return index   
   
#########################
# MISC HELPER FUNCTIONS #
#########################            
def SortResultsAndSetKey(genedic,key="p_v"):
    """
    A method for returning a dictionary where genes are sorted by 
    p-value
    """
    return {i: gene for (i, gene) in enumerate(sorted(genedic,
                                       key=lambda x: genedic[x][key])) }
                                                      
def SortResultsAndSetKeyPairwise(genedic,correctionmethod):
    """
    A method for returning a dictionary where genes are sorted by 
    pairwise comparison p-values
    """
    if "EPW" in correctionmethod:
        return {i: gene for (i, gene) in enumerate(sorted(genedic,
                                   key=lambda x: genedic[x]["Pboth"])) }

    elif "PW" in correctionmethod:
        return {i: gene for (i, gene) in enumerate(sorted(genedic,
                                 key=lambda x: genedic[x]["Plowest"])) }
    else:
        sys.exit("Something went wrong when using this set of "
        "correction methods. Please report this bug")
                                                      
def filtrationoptions(cutoffs, collapse):
    """
    Converts between easy keys (I, B, etc) and long names of filtration 
    options, for printing to the runlog
    """
    translation = {"I": "Individual (Naive)", "B": "Bonferroni", 
                   "BH":"Benjamini-Hochberg",
                   "PW":"Pairwise comparison (Best)", 
                   "EPW": "Pairwise comparison (Entire range)",
                   "P": "Empirical p-value (permutation-based)"}
    filters = [str(translation[k]) + ":    " + str(v)
               for k,v in cutoffs.items()] 
    filters.append("Collapse genes:    " + str(collapse) + "\n\n")
    filters = ["-- Filtration options --"] + filters

    return filters
    
def decideifbreak(cutoffs, currentgene):
    """
    Decides if the loop should continue or if all filtration thresholds 
    are violated and therefore should break.
    """
    if (("I" in cutoffs) or ("B" in cutoffs) or ("BH" in cutoffs)):
        if "I" in cutoffs and currentgene["p_v"] > cutoffs["I"]:
            return True
        elif "B" in cutoffs and currentgene["B_p"] > cutoffs["B"]:
            return True
        elif "BH" in cutoffs and currentgene["BH_p"] > cutoffs["BH"]:
            return True
        else:
            # All p-values are too low. Cannot break yet.
            return False
    else:
        # Can't break early if none of the basic filtrations are used, 
        # because there might be other genes downstream that pass
        # pairwise and empirical filtration 
        return False

def grabcoltype(string):
    """
    Accepts as arguments numbers with space, commas and range and
    returns a list of all numbers
    """
    # If the keyword ALL is passed, return all input columns that have
    # not been restricted from --restrict_to.
    if string == "ALL":
        return [-999]
    if string == "":
        return []
    bycommas = string.split(",")

    returnlist = []
    for s in bycommas:
        if "-" in s:
            #Expand range
            try:
                bydash = s.split("-")
                if not (int(bydash[1]) > int(bydash[0])):
                    raise
                add = range(int(bydash[0]), int(bydash[1]))
                returnlist += list(add)
            except:
                sys.exit("Could not understand --include_input_columns "
                         "argument %s" % s)
        else:
            try:
                returnlist += [int(s)]
            except TypeError:
                sys.exit("Could not understand --include_input_columns "
                         "argument %s" % s)
    #Adjust to 0-based indexing
    returnlist = [i-1 for i in returnlist]
    #Remove the columns 0, 1 and 2, since they are always included in the output
    returnlist = [i for i in returnlist if i not in [0,1,2]]
    if not all([i > 1 for i in returnlist]):
        sys.exit("Could not understand --include_input_columns argument. "
                 "Make sure all numbers are positive and real.")
    return list(set(returnlist))

def ScoaryArgumentParser():
    """
    Function for parsing the arguments separate from the main function.
    Makes it easier for other scripts to run Scoary without artificially
    constructing a command line
    """
    # Parse arguments.
    parser = argparse.ArgumentParser(
            description='Scoary version %s - Screen pan-genome for '
                        'trait-associated variants' % SCOARY_VERSION,
                        epilog='by Ola Brynildsrud (olbb@fhi.no)')
    
    inputgroup = parser.add_argument_group('Input options')
    inputgroup.add_argument('-t', '--traits',
                        help='Input trait table '
                        '(comma-separated-values). Trait presence is '
                        'indicated by 1, trait absence by 0. Assumes '
                        'strain names in the first column and trait '
                        'names in the first row')
    inputgroup.add_argument('-g', '--genes',
                        help='Input gene presence/absence table '
                        '(comma-separated-values) from ROARY. '
                        'Strain names must be equal to those in the '
                        'trait table')
    inputgroup.add_argument('-n', '--newicktree',
                        help='Supply a custom tree (Newick format) for '
                        'phylogenetic analyses instead instead of '
                        'calculating it internally.',
                        default=None)
    inputgroup.add_argument('-s', '--start_col',
                        help='On which column in the gene '
                        'presence/absence file do individual strain '
                        'info start. Default=15. (1-based indexing)',
                        default=15,
                        type=int)
    inputgroup.add_argument('--delimiter',
                        help='The delimiter between cells in the gene '
                        'presence/absence and trait files, as well as '
                        'the output file. ',
                        default=',',
                        type=str)
    inputgroup.add_argument('-r', '--restrict_to',
                        help='Use if you only want to analyze a subset '
                        'of your strains. Scoary will read the '
                        'provided comma-separated table of strains and '
                        'restrict analyzes to these.')

    outputgroup = parser.add_argument_group('Output options')
    outputgroup.add_argument('-o', '--outdir',
                        help='Directory to place output files.'
                        ' Default = .',
                        default='./')
    outputgroup.add_argument('-u', '--upgma_tree',
                        help='This flag will cause Scoary to write the '
                        'calculated UPGMA tree to a newick file',
                        default=False,
                        action='store_true')
    outputgroup.add_argument('-p', '--p_value_cutoff',
                        help='P-value cut-off / alpha level. '
                        'For Fishers, Bonferronis, and '
                        'Benjamini-Hochbergs tests, SCOARY will not '
                        'report genes with higher p-values than this. '
                        'For empirical p-values, this is treated as an '
                        'alpha level instead. I.e. 0.02 will filter '
                        'all genes except the lower and upper '
                        'percentile from this test. Run with "-p 1.0" '
                        'to report all genes. Accepts standard form '
                        '(e.g. 1E-8). Provide a single value (applied '
                        'to all) or exactly as many values as '
                        'correction criteria and in corresponding '
                        'order. (See example under correction). '
                        'Default = 0.05',
                        nargs='+',
                        default=[0.05],
                        type=float)
    cutoffchoices = ['I', 'B', 'BH','PW','EPW','P']
    outputgroup.add_argument('-c', '--correction',
                        help='Apply the indicated filtration measure. '
                        'Allowed values are %s. '
                        'I=Individual (naive) p-value. '
                        'B=Bonferroni adjusted p-value. '
                        'BH=Benjamini-Hochberg adjusted p. '
                        'PW=Best (lowest) pairwise comparison. '
                        'EPW=Entire range of pairwise comparison '
                        'p-values. P=Empirical p-value from '
                        'permutations. You can enter as many '
                        'correction criteria as you would like. These '
                        'will be associated with the p_value_cutoffs '
                        'you enter. For example "-c I EPW -p 0.1 0.05" '
                        'will apply the following cutoffs: Naive '
                        'p-value must be lower than 0.1 AND the entire '
                        'range of pairwise comparison values are below '
                        '0.05 for this gene. Note that the empirical '
                        'p-values should be interpreted at both tails. '
                        'Therefore, running "-c P -p 0.05" will apply '
                        'an alpha of 0.05 to the empirical (permuted) '
                        'p-values, i.e. it will filter everything '
                        'except the upper and lower 2.5 percent of the '
                        'distribution. '
                        'Default = Individual p-value. (I)' % ', '.join(
                        cutoffchoices),
                        choices=cutoffchoices,
                        nargs='*',
                        default=['I'])
    outputgroup.add_argument('-m', '--max_hits',
                        help='Maximum number of hits to report. SCOARY '
                        'will only report the top max_hits results per '
                        'trait',
                        type=int)
    outputgroup.add_argument('--include_input_columns',
                        help='Grab columns from the input Roary file. '
                        'and puts them in the output. Handles comma '
                        'and ranges, e.g. --include_input_columns '
                        '4,6,8,16-23. The special keyword ALL will include '
                        'all relevant input columns in the output',
                        dest='grabcols',
                        type=grabcoltype,
                        default=[])
    outputgroup.add_argument('-w', '--write_reduced',
                        help='Use with -r if you want Scoary to create '
                        'a new gene presence absence file from your '
                        'reduced set of isolates. Note: Columns 1-14 '
                        '(No. sequences, Avg group size nuc etc) in '
                        'this file do not reflect the reduced dataset. '
                        'These are taken from the full dataset.',
                        default=False,
                        action='store_true')
    outputgroup.add_argument('--no-time',
                        help='Output file in the form '
                        'TRAIT.results.csv, instead of '
                        'TRAIT_TIMESTAMP.csv. When used with the -w '
                        'argument will output a reduced gene matrix in '
                        'the form gene_presence_absence_reduced.csv '
                        'rather than '
                        'gene_presence_absence_reduced_TIMESTAMP.csv ',
                        default=False,
                        action='store_true')

    analysisgroup = parser.add_argument_group('Analysis options')
    analysisgroup.add_argument('-e','--permute',
                        help='Perform N number of permutations of the '
                        'significant results post-analysis. Each '
                        'permutation will do a label switching of the '
                        'phenotype and a new p-value is calculated '
                        'according to this new dataset. After all N '
                        'permutations are completed, the results are '
                        'ordered in ascending order, and the '
                        'percentile of the original result in the '
                        'permuted p-value distribution is reported.',
                        type=int,
                        default=0)
    analysisgroup.add_argument('--no_pairwise',
                        help='Do not perform pairwise comparisons. In'
                        'this mode, Scoary will perform population '
                        'structure-naive calculations only. (Fishers '
                        'test, ORs etc). Useful for summary operations '
                        'and exploring sets. (Genes unique in groups, '
                        'intersections etc) but not causal analyses.',
                        default=False,
                        action='store_true')
    analysisgroup.add_argument('--collapse',
                        help='Add this to collapse correlated genes '
                        '(genes that have identical distribution '
                        'patterns in the sample) into merged units. ',
                        default=False,
                        action='store_true')

    miscgroup = parser.add_argument_group('Misc options')
    miscgroup.add_argument('--threads',
                        help='Number of threads to use. Default = 1',
                        type=int,
                        default=1)
    miscgroup.add_argument('--test',
                        help='Run Scoary on the test set in '
                        'exampledata, overriding all other parameters.',
                        default=False,
                        action='store_true')
    miscgroup.add_argument('--citation',
                        help='Show citation information, and exit. ',
                        default=False,
                        action='store_true')
    miscgroup.add_argument('--version', help='Display Scoary version, and '
                        'exit.',
                        action='version',
                        version=SCOARY_VERSION)

    args = parser.parse_args()

    if len(args.p_value_cutoff) == 1:
        cutoffs = {c : args.p_value_cutoff[0] for c in args.correction}
    else:
        cutoffs = dict(list(zip(args.correction, args.p_value_cutoff)))
        
    return args, cutoffs

########
# MAIN #
########
if __name__ == '__main__':
    pass



import sys

try:
    from ete3 import Tree
    from ete3.parser.newick import NewickError
    import six
except ImportError:
    sys.exit("ERROR: Could not import ete3. You need 'ete3' AND 'six' installed to read custom trees.")

def ReadTreeFromFile(filepath):
    """
    Uses ete3 to read a newick tree file, and converts this to a Scoary-readable nested list
    """
    try:
        myTree = Tree(filepath)
    except NewickError as e:
        sys.exit("Corrupted or non-existing custom tree file? %s" % e)
        
    myTree.resolve_polytomy(recursive=True)
    myTreeList, members = RecTree2List(myTree,Members=None)

    return myTreeList, members

def RecTree2List(Tree, Members=None):
    """
    Recursive function that at each node create a list of the children nodes. Can be nested
    """
    List = []
    if Members is None:
        Members = []

    if len(Tree._children) == 0:
        shavedname = Tree.name.lstrip("'\"").rstrip("'\"")
        Members += [shavedname]
        return str(shavedname), Members
    else:
        for node in Tree._children:
            mynode, Members = RecTree2List(node, Members)
            List.append(mynode)
        return List, Members




from setuptools import setup
from scoary import __version__ as sv

setup(name='scoary',
      version=sv,
      description='Microbial pan-GWAS using the output from Roary',
      classifiers=[
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Programming Language :: Python :: 2.7',
        'Topic :: Scientific/Engineering :: Bio-Informatics',
        'Topic :: Scientific/Engineering :: Medical Science Apps.',
        'Intended Audience :: Science/Research',
      ],
      keywords=['microbial', 'genomics', 'GWAS', 'Roary'],
      url='https://github.com/AdmiralenOla/Scoary',
      download_url=('https://github.com/AdmiralenOla/Scoary/tarball/v%s' % sv),
      author='Ola Brynildsrud',
      author_email='ola.brynildsrud@fhi.no',
      license='GPLv3',
      packages=['scoary'],
      install_requires=[
          'scipy>=0.16',
          'argparse'
      ],
      test_suite='nose.collector',
      tests_require=[],
      entry_points={
          'console_scripts': ['scoary=scoary.methods:main',
                             'scoary_GUI=scoary.GUI:main',
                             'vcf2scoary=scoary.vcf2scoary:main'],
      
      },
      include_package_data=True,
      zip_safe=False)
